/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include "naap_am.h"
#include "naap_dt.h"
#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <string.h>

extern char* debuglogfile;

/* the debug tracing function used by the NAAP Core */
void dt(u_int32_t zone, const u_int8_t* pszFormat, ...)
{
  char buf[65536] = {0};
  va_list arglist;
  FILE *fp;
  time_t t;
  char *timestr;
  
  u_int32_t current_zone_mask = 0xFFFFFFFF; /* TODO: actual value */

  if (!(zone & current_zone_mask))
    {
      return;
    }

  t = time(NULL);
  timestr = ctime(&t);
  timestr[strlen(timestr) - 1] = '\0';

  va_start(arglist, pszFormat);
  vsprintf(buf, pszFormat, arglist);
  va_end(arglist);


  if (debuglogfile)
  {
      fp = fopen(debuglogfile, "a+");

      if (fp) 
      {
           fprintf(fp, "%s: ", timestr);
           fprintf(fp, buf);
           fflush(fp);
           fclose(fp);
      }
  }
  
  return;
}
 
void dt_array(u_int32_t zone, const u_int8_t* header_text, const u_int8_t* line_header, 
              u_int8_t *array, u_int32_t array_len, u_int8_t output_mode)
{
  FILE *fp;
  time_t t;
  char *timestr;
  u_int32_t current_zone_mask = 0xFFFFFFFF; /* TODO: actual value */
  u_int32_t i = 0;

  if (!(zone & current_zone_mask))
    {
      return;
    }

  t = time(NULL);
  timestr = ctime(&t);
  timestr[strlen(timestr) - 1] = '\0';

  if (!debuglogfile)
  {
       fp = fopen(debuglogfile, "a+");
       
       if (fp) 
       {
            fprintf(fp, "%s: ", timestr);
            if (header_text)
	    {
                 fprintf(fp, "%s", header_text);
	    }
            for (i = 1; i <= array_len; i++)
	    {
                 if (output_mode == ARRAY_OUTPUT_HEX)
                 {
                      fprintf(fp, "%.2X ", array[i - 1]);
                 }
                 else
                 {
                      fprintf(fp, "%c", array[i - 1]);
                 }
                 if ((i % 4) == 0)
                 {
                      if (output_mode == ARRAY_OUTPUT_HEX)
                      {
                           fprintf(fp, "  ");
                      }
                      if ((i % 16) == 0 && i < array_len)
                      {
                           fprintf(fp, "\n");
                           fprintf(fp, "%s: ", timestr);
                           if (line_header)
                           {
                                fprintf(fp, "%s", line_header);
                           }
                      }
                 }
	    }
            fprintf(fp, "\n");
            fflush(fp);
            fclose(fp);
       }
  }
  
  return;
}
