/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include "naap_am.h"
#include "am_fileconfig_utils.h"
#include "naap_dt.h"
#include "naap_conf.h"
#include "am_global.h"
#include "naap_socket.h"

static void cnf_address(char* line, void* data)
{   
     am_own_ip_addr = cnf_get_addr(line);
}

static void cnf_mode(char* line, void* data)
{   
     /* "x" is a dummy value, needed to keep mode as UNINITIALIZED (0) if 
      * the file contains an invalid mode 
      */
     char const *modes [] = {"x", "agent", "client", NULL}; 
     
     am_naap_mode = cnf_get_enum(line, modes);
}

static void cnf_solicitation_interval(char* line, void* data)
{
     nc_conf_solicitation_interval((u_int16_t)cnf_get_int32(line));
}

static void cnf_max_solicitation_interval(char* line, void* data)
{
     nc_conf_max_solicitation_interval((u_int16_t)cnf_get_int32(line));
}

static void cnf_sim_timeout(char* line, void* data)
{
     nc_conf_sim_timeout((u_int16_t)cnf_get_int32(line));
}

static void cnf_id_retransmission_timeout(char* line, void* data)
{
     nc_conf_id_retransmission_timeout((u_int16_t)cnf_get_int32(line));
}

static void cnf_max_id_retransmissions(char* line, void* data)
{
     nc_conf_max_id_retransmissions((u_int16_t)cnf_get_int32(line));
}

static void cnf_start_retransmission_timeout(char* line, void* data)
{
     nc_conf_start_retransmission_timeout((u_int16_t)cnf_get_int32(line));
}

static void cnf_max_start_retransmissions(char* line, void* data)
{
     nc_conf_max_start_retransmissions((u_int16_t)cnf_get_int32(line));
}

static void cnf_challenge_retransmission_timeout(char* line, void* data)
{
     nc_conf_challenge_retransmission_timeout((u_int16_t)cnf_get_int32(line));
}

static void cnf_max_challenge_retransmissions(char* line, void* data)
{
     nc_conf_max_challenge_retransmissions((u_int16_t)cnf_get_int32(line));
}

static void cnf_default_reg_lifetime(char* line, void* data)
{
     nc_conf_default_reg_lifetime((u_int16_t)cnf_get_int32(line));
}

static void cnf_reg_refresh_time(char* line, void* data)
{
     nc_conf_reg_refresh_time((u_int16_t)cnf_get_int32(line));
}

static void cnf_local_request_retransmission_timeout(char* line, void* data)
{
     nc_conf_local_request_retransmission_timeout((u_int16_t)cnf_get_int32(line));
}

static void cnf_max_local_request_retransmissions(char* line, void* data)
{
     nc_conf_max_local_request_retransmissions((u_int16_t)cnf_get_int32(line));
}

static void cnf_default_sim_key_lifetime(char* line, void* data)
{
     nc_conf_default_sim_key_lifetime((u_int16_t)cnf_get_int32(line));
}

static void cnf_reauthentication_time(char* line, void* data)
{
     nc_conf_reauthentication_time((u_int16_t)cnf_get_int32(line));
}

static void cnf_realm(char* line, void* data)
{
     cnf_get_string(am_realm, line, AM_MAX_REALM_LENGTH + 1);
}

void config_general(FILE* file)
{
     SubSection subsections [] = {
          {"address", cnf_address},
          {"mode", cnf_mode},
          {"solicitation_interval", cnf_solicitation_interval},
          {"max_solicitation_interval", cnf_max_solicitation_interval},
          {"sim_timeout", cnf_sim_timeout},
          {"id_retransmission_timeout", cnf_id_retransmission_timeout},
          {"max_id_retransmissions", cnf_max_id_retransmissions},
          {"start_retransmission_timeout", cnf_start_retransmission_timeout},
          {"max_start_retransmissions", cnf_max_start_retransmissions},
          {"challenge_retransmission_timeout", cnf_challenge_retransmission_timeout},
          {"max_challenge_retransmissions", cnf_max_challenge_retransmissions},
          {"default_reg_lifetime", cnf_default_reg_lifetime},
          {"reg_refresh_time", cnf_reg_refresh_time},
          {"local_request_retransmission_timeout", cnf_local_request_retransmission_timeout},
          {"max_local_request_retransmissions", cnf_max_local_request_retransmissions},
          {"default_sim_key_lifetime", cnf_default_sim_key_lifetime},
          {"reauthentication_time", cnf_reauthentication_time},
          {"realm", cnf_realm},
          { NULL, NULL }
     };
     
     cnf_subsects (file, subsections, NULL);
}

void fileconfig_core(char *filename)
{
     FILE* cnf;
        
     Section sections [] = {
          { "[naap]", config_general },
          { NULL, NULL } 
     };

     am_naap_mode = NAAP_CLIENT;
     nc_conf_solicitation_interval(30);
     nc_conf_max_solicitation_interval(600);
     nc_conf_sim_timeout(30);
     nc_conf_id_retransmission_timeout(5);
     nc_conf_max_id_retransmissions(5);
     nc_conf_start_retransmission_timeout(5);
     nc_conf_max_start_retransmissions(5);
     nc_conf_challenge_retransmission_timeout(5);
     nc_conf_max_challenge_retransmissions(5);
     nc_conf_default_reg_lifetime(85);
     nc_conf_reg_refresh_time(15);
     nc_conf_local_request_retransmission_timeout(3);
     nc_conf_max_local_request_retransmissions(7);
     nc_conf_default_sim_key_lifetime(14400);
     nc_conf_reauthentication_time(60);
     cnf_get_string(am_realm, "realm = owlan.org", AM_MAX_REALM_LENGTH + 1);

     /* This should be OK because we should have default settings. */
     if (!filename)
          return;

     cnf = fopen (filename, "r");
     
     if (!cnf) {
          DT((DT_CONF, "ERROR: fileconfig_core: configuration file %s not found\n", filename));
          exit(2);
     }
     
     config_sections (cnf, sections);
     
     fclose(cnf);
}
