/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <time.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <naap_am.h>
#include <naap_event.h>
#include <naap_ui.h>
#include <naap_dt.h>
#include "am_global.h"
#include "am_gui.h"
#include "operator_icon.xpm"

static AuthenticationState m_State;

/* Flag to indicate whether the terminal is authenticated */
static bool m_Connected;
/* Flag to indicate whether a timeout has occurred during authentication */
static bool m_Timeout;
/* Flag to indicate whether an error has occurred during authentication */
static bool m_OperationFailed;

static GTimer *main_timer = NULL;
static GtkWidget *main_dialog;
static GtkWidget *main_button_connect;
static GtkWidget *connect_dialog;
static GtkWidget *connect_progressbar;
static GtkWidget *main_label_status;
static GtkWidget *main_label_duration;
static GtkWidget *history_dialog;
static GtkWidget *history_clist;
extern int stop_flag;

gint gui_delete_event(GtkWidget *widget,
                      GdkEvent *event,
                      gpointer data )
{
#ifdef DEEP_DEBUG
     g_print ("gui_delete_event() entered.\n");
#endif /* DEEP_DEBUG */

     return FALSE;
}

void gui_destroy(GtkWidget *widget,
                 gpointer data )
{
#if DEEP_DEBUG
     g_print ("gui_destroy() entered.\n");
#endif /* DEEP_DEBUG */

     if (widget == main_dialog) {
#if DEEP_DEBUG
          g_print("killing main dialog!\n");
#endif /* DEEP_DEBUG */
          main_dialog = NULL;
          stop_flag = 1;
          return;
     }

     if (widget == connect_dialog) {
#if DEEP_DEBUG
          g_print("killing connect dialog!\n");
#endif /* DEEP_DEBUG */
          connect_dialog = NULL;
          return;
     }
}

void gui_connect_button_enable(void)
{
     gtk_widget_set_sensitive(main_button_connect, TRUE);

}

void gui_connect_button_disable(void)
{
     gtk_widget_set_sensitive(main_button_connect, TRUE);
}

void gui_connect_button_set_text(char *text)
{
     gtk_label_set_text(GTK_LABEL(GTK_BIN(main_button_connect)->child), text); 
}

void gui_update_connect_progressbar(gfloat percentage) 
{
     if (!connect_progressbar)
          return;
     
     gtk_progress_bar_update(GTK_PROGRESS_BAR(connect_progressbar), 
                             percentage);
}

void gui_update_status(char *statustext)
{
     gtk_label_set_text(GTK_LABEL(main_label_status), statustext);
}

void gui_update_duration(gdouble seconds)
{
     unsigned int hours, minutes, secs;
     char duration[10];
     
     hours = (unsigned int)seconds / 3600;
     minutes = ((unsigned int)seconds - (3600 * hours)) / 60;
     secs = ((unsigned int)seconds - (3600 * hours)) - (60 * minutes);

     sprintf(duration, "%02u:%02u:%02u\n", hours, minutes, secs);

     gtk_label_set_text(GTK_LABEL(main_label_duration), duration);
}

void gui_about_clicked(GtkWidget *widget, GtkWidget* dialog) 
{
     gtk_widget_destroy(dialog);
}

void gui_create_about()
{
     GtkWidget *about_dialog;
     GtkWidget *about_button_ok;
     GtkWidget *about_message;
     
     about_dialog = gtk_dialog_new();

     gtk_window_set_title (GTK_WINDOW(about_dialog), IDS_INFO_CAPTION);

     about_button_ok = gtk_button_new_with_label("OK");

     gtk_signal_connect(GTK_OBJECT(about_button_ok), "clicked",
                        GTK_SIGNAL_FUNC(gui_about_clicked), about_dialog);

     gtk_container_set_border_width (GTK_CONTAINER (about_dialog), 10);

     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->action_area),
                         about_button_ok, TRUE, TRUE, 0);
     
     about_message = gtk_label_new("Operator X Access Suite\n\nCopyright (C) 2002 Operator X");
     
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->vbox),
                         about_message, TRUE, TRUE, 0);
     
     gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (about_dialog)->vbox), 5);
     
     gtk_widget_show_all(about_dialog);
}

void gui_history_window_toggle()
{
     static short int flag = 1;

     if (!flag) {
          gtk_widget_hide_all(history_dialog);
          flag = 1;
     } else {
          gtk_widget_show_all(history_dialog);
          flag = 0;
     }
}

void gui_history_add_event(gchar *date, gchar *time, gchar *event)
{
     gchar *clist_text[3] = { date, time, event };

     gtk_clist_append(GTK_CLIST(history_clist), clist_text);
}

void gui_history_set_log_message(gchar *event)
{
     time_t current_time;
     struct tm *t;
     char datestr[12], timestr[10];

     current_time = time(NULL);

     t = localtime(&current_time);
     strftime(datestr, 12, "%d-%m-%Y", t);
     strftime(timestr, 10, "%H:%M:%S", t);

     gui_history_add_event(datestr, timestr, event);
}

void gui_history_close_clicked(GtkWidget *widget, gpointer data) 
{
     gui_history_window_toggle();
}

void gui_create_history(void)
{
     GtkWidget *vbox, *hbox;
     GtkWidget *scrolled_window;
     GtkWidget *button_close;
     gchar *titles[3] = { "Date", "Time", "Event" };
     
     history_dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_widget_set_usize(GTK_WIDGET(history_dialog), 400, 300);
     gtk_window_set_title(GTK_WINDOW(history_dialog), IDS_INFO_CAPTION);
     gtk_signal_connect(GTK_OBJECT(history_dialog), "delete_event",
                        GTK_SIGNAL_FUNC(gui_delete_event), gui_history_close_clicked);

     vbox = gtk_vbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
     gtk_container_add(GTK_CONTAINER(history_dialog), vbox);

     scrolled_window = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
     gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

     history_clist = gtk_clist_new_with_titles(3, titles);
     gtk_clist_set_shadow_type(GTK_CLIST(history_clist), GTK_SHADOW_OUT);
     gtk_clist_set_column_width(GTK_CLIST(history_clist), 0, 75);
     gtk_clist_set_column_width(GTK_CLIST(history_clist), 1, 55);
     gtk_clist_set_column_width(GTK_CLIST(history_clist), 2, 200);
     gtk_container_add(GTK_CONTAINER(scrolled_window), history_clist);

     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);

     button_close = gtk_button_new_with_label("Close");
     gtk_box_pack_start(GTK_BOX(hbox), button_close, TRUE, TRUE, 0);
     gtk_signal_connect(GTK_OBJECT (button_close), "clicked",
			GTK_SIGNAL_FUNC(gui_history_close_clicked), NULL);
}

void gui_yesno_reauth_clicked(GtkWidget *widget, gpointer data) {
     GtkWidget *dialog = gtk_object_get_data(GTK_OBJECT(widget), "dialog");

     gtk_widget_destroy(GTK_WIDGET(dialog));
     m_State = Reauthentication;
}

void gui_yesno_cancel_clicked(GtkWidget *widget, gpointer data) {
     GtkWidget *dialog = gtk_object_get_data(GTK_OBJECT(widget), "dialog");

     gtk_widget_destroy(GTK_WIDGET(dialog));
}

void gui_create_yesno(const gchar *msg, void (*yescb)(GtkWidget *, gpointer),
                      void (*nocb)(GtkWidget *, gpointer), gpointer data)
{
     GtkWidget *dialog, *widget;
     
     dialog = gtk_dialog_new();
     gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
     gtk_window_set_title(GTK_WINDOW(dialog), "Confirmation");
     gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
     gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
                        GTK_SIGNAL_FUNC(gui_delete_event), dialog);

     widget = gtk_label_new(msg);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), widget,
                        TRUE, TRUE, 5);
     gtk_widget_show(widget);
     
     widget = gtk_button_new_with_label("OK");
     gtk_signal_connect(GTK_OBJECT(widget), "clicked",
                        GTK_SIGNAL_FUNC(yescb), data);
     gtk_object_set_data(GTK_OBJECT(widget), "dialog", dialog);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), widget,
                        TRUE, TRUE, 5);
     gtk_widget_show(widget);
     
     widget = gtk_button_new_with_label("Cancel");
     if(nocb)
          gtk_signal_connect(GTK_OBJECT(widget), "clicked",
                             GTK_SIGNAL_FUNC(nocb), data);
     else
          gtk_signal_connect(GTK_OBJECT(widget), "clicked",
                             GTK_SIGNAL_FUNC(gui_yesno_cancel_clicked), NULL);
     gtk_object_set_data(GTK_OBJECT(widget), "dialog", dialog);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), widget,
                        TRUE, TRUE, 5);
     gtk_widget_show(widget);
     
     gtk_widget_show(dialog);
}

void gui_error_clicked(GtkWidget *widget, 
                       GtkWidget *dialog ) 
{
     gtk_widget_destroy(dialog);
}

void gui_create_error(char* error)
{
     GtkWidget *error_dialog;
     GtkWidget *error_button_ok;
     GtkWidget *error_message;
     
     error_dialog = gtk_dialog_new();

     gtk_window_set_title (GTK_WINDOW(error_dialog), IDS_ERROR_CAPTION);

     error_button_ok = gtk_button_new_with_label("OK");

     gtk_signal_connect(GTK_OBJECT(error_button_ok), "clicked",
                        GTK_SIGNAL_FUNC(gui_error_clicked), error_dialog);

     gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 10);

     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->action_area),
                         error_button_ok, TRUE, TRUE, 0);
     
     error_message = gtk_label_new(error);
     
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->vbox),
                         error_message, TRUE, TRUE, 0);
     
     gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (error_dialog)->vbox), 5);
     
     gtk_widget_show_all(error_dialog);
}

void gui_connect_clicked(GtkWidget *widget, 
                         gpointer data)
{
#ifdef DEEP_DEBUG
     g_print("gui_connect() entered.\n");
#endif /* DEEP_DEBUG */

     if (!m_Connected)
          nc_connect_clicked();
     else
          nc_disconnect_clicked();
}

void gui_create_connect(void)
{
     assert(connect_dialog == NULL);

     connect_dialog = gtk_window_new(GTK_WINDOW_DIALOG);

     gtk_window_set_title(GTK_WINDOW(connect_dialog), "Connection Status");

     gtk_container_set_border_width(GTK_CONTAINER(connect_dialog), 10);

     gtk_signal_connect(GTK_OBJECT(connect_dialog), "destroy",
                        GTK_SIGNAL_FUNC(gui_destroy), NULL);

     connect_progressbar = gtk_progress_bar_new();

     gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(connect_progressbar),
                                      GTK_PROGRESS_LEFT_TO_RIGHT);

     gtk_container_add(GTK_CONTAINER(connect_dialog), connect_progressbar);

     gtk_widget_show_all(connect_dialog);
}

void gui_delete_connect(void)
{
     if (!connect_dialog)
          return;

     gtk_widget_destroy(connect_dialog);
}

void gui_menu_exit1_activate(GtkWidget *widget, gpointer data) 
{
     stop_flag = 1;
}

void gui_menu_about1_activate(GtkWidget *widget, gpointer data) 
{
     gui_create_about();
}

void gui_menu_history1_activate(GtkWidget *widget, gpointer data) 
{
     gui_history_window_toggle();
}

void gui_create_main(void)
{
  GtkWidget *vbox1;
  GtkWidget *menu;
  guint tmp_key;
  GtkWidget *file1;
  GtkWidget *file1_menu;
  GtkAccelGroup *file1_menu_accels;
  GtkWidget *exit1;
  GtkWidget *history1;
  GtkWidget *help1;
  GtkWidget *help1_menu;
  GtkAccelGroup *help1_menu_accels;
  GtkWidget *about1;
  GtkWidget *hbox2;
  GtkWidget *table1;
  GtkWidget *label_statustext;
  GtkWidget *label_durationtext;
  GtkWidget *operator;
  GtkWidget *hseparator1;
  GtkWidget *label_note;
  GdkPixmap* icon;
  GdkBitmap* icon_mask;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  main_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (main_dialog), "main_dialog", main_dialog);
  gtk_window_set_title (GTK_WINDOW (main_dialog), IDS_INFO_CAPTION);

  vbox1 = gtk_vbox_new (FALSE, 10);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (main_dialog), vbox1);

  menu = gtk_menu_bar_new ();
  gtk_widget_ref (menu);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "menu", menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menu);
  gtk_box_pack_start (GTK_BOX (vbox1), menu, FALSE, FALSE, 0);

  file1 = gtk_menu_item_new_with_label ("");
  tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (file1)->child),
                                   "_File");
  gtk_widget_add_accelerator (file1, "activate_item", accel_group,
                              tmp_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_ref (file1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "file1", file1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (file1);
  gtk_container_add (GTK_CONTAINER (menu), file1);

  file1_menu = gtk_menu_new ();
  gtk_widget_ref (file1_menu);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "file1_menu", file1_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file1), file1_menu);
  file1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (file1_menu));

  exit1 = gtk_menu_item_new_with_label ("");
  tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (exit1)->child),
                                   "_Quit");
  gtk_widget_add_accelerator (exit1, "activate_item", file1_menu_accels,
                              tmp_key, 0, 0);
  gtk_widget_ref (exit1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "exit1", exit1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (exit1);
  gtk_container_add (GTK_CONTAINER (file1_menu), exit1);

  history1 = gtk_menu_item_new_with_label ("");
  tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (history1)->child),
                                   "H_istory");
  gtk_widget_add_accelerator (history1, "activate_item", accel_group,
                              tmp_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_ref (history1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "history1", history1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (history1);
  gtk_container_add (GTK_CONTAINER (menu), history1);

  help1 = gtk_menu_item_new_with_label ("");
  tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (help1)->child),
                                   "_Help");
  gtk_widget_add_accelerator (help1, "activate_item", accel_group,
                              tmp_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
  gtk_widget_ref (help1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "help1", help1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (help1);
  gtk_container_add (GTK_CONTAINER (menu), help1);

  help1_menu = gtk_menu_new ();
  gtk_widget_ref (help1_menu);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "help1_menu", help1_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (help1), help1_menu);
  help1_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (help1_menu));

  about1 = gtk_menu_item_new_with_label ("");
  tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (about1)->child),
                                   "_About");
  gtk_widget_add_accelerator (about1, "activate_item", help1_menu_accels,
                              tmp_key, 0, 0);
  gtk_widget_ref (about1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "about1", about1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (help1_menu), about1);

  hbox2 = gtk_hbox_new (FALSE, 10);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 10);

  table1 = gtk_table_new (3, 2, TRUE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (hbox2), table1, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 10);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 30);

  label_statustext = gtk_label_new ("Status:");
  gtk_widget_ref (label_statustext);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "label_statustext", label_statustext,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_statustext);
  gtk_table_attach (GTK_TABLE (table1), label_statustext, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label_statustext), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label_statustext), 0, 0.5);

  label_durationtext = gtk_label_new ("Duration:");
  gtk_widget_ref (label_durationtext);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "label_durationtext", label_durationtext,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_durationtext);
  gtk_table_attach (GTK_TABLE (table1), label_durationtext, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label_durationtext), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label_durationtext), 0, 0.5);

  main_label_status = gtk_label_new ("Service not available");
  gtk_widget_ref (main_label_status);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "main_label_status", main_label_status,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (main_label_status);
  gtk_table_attach (GTK_TABLE (table1), main_label_status, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (main_label_status), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (main_label_status), TRUE);
  gtk_misc_set_alignment (GTK_MISC (main_label_status), 0, 0.5);

  main_label_duration = gtk_label_new ("00:00:00");
  gtk_widget_ref (main_label_duration);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "main_label_duration", main_label_duration,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (main_label_duration);
  gtk_table_attach (GTK_TABLE (table1), main_label_duration, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (main_label_duration), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (main_label_duration), 0, 0.5);

  main_button_connect = gtk_button_new_with_label ("Connect");
  gtk_widget_ref (main_button_connect);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "main_button_connect", main_button_connect,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (main_button_connect);
  gtk_table_attach (GTK_TABLE (table1), main_button_connect, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_widget_realize(GTK_WIDGET(main_dialog));

  icon = gdk_pixmap_create_from_xpm_d(main_dialog->window, &icon_mask, 
                                      NULL, operator_icon_xpm);
  operator = gtk_pixmap_new(icon, icon_mask);
     
  gtk_widget_ref (operator);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "operator", operator,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (operator);
  gtk_box_pack_start (GTK_BOX (hbox2), operator, TRUE, TRUE, 0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);

  label_note = gtk_label_new ("The actual invoice for services from your service provider may vary, depending upon network features, rounding-off for billing, taxes and so forth.");
  gtk_widget_ref (label_note);
  gtk_object_set_data_full (GTK_OBJECT (main_dialog), "label_note", label_note,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label_note);
  gtk_box_pack_start (GTK_BOX (vbox1), label_note, FALSE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label_note), TRUE);
  gtk_misc_set_padding (GTK_MISC (label_note), 0, 10);

  gtk_signal_connect (GTK_OBJECT (main_dialog), "delete_event",
                      GTK_SIGNAL_FUNC (gui_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (main_dialog), "destroy",
                      GTK_SIGNAL_FUNC (gui_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (exit1), "activate",
                      GTK_SIGNAL_FUNC (gui_menu_exit1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (history1), "activate",
                      GTK_SIGNAL_FUNC (gui_menu_history1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (about1), "activate",
                      GTK_SIGNAL_FUNC (gui_menu_about1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (main_button_connect), "clicked",
                      GTK_SIGNAL_FUNC (gui_connect_clicked),
                      NULL);

  gtk_window_add_accel_group (GTK_WINDOW (main_dialog), accel_group);

  gtk_widget_show_all(main_dialog);
}

void gui_handle_operation_failed(int arg)
{
     char message[IDS_ERROR_MAX_LENGTH];
     strcpy(message, "");

     switch (arg)
     {
     case PROTERR_SIM_FAILURE:
          strncpy(message, IDS_SIM_ERROR, IDS_ERROR_MAX_LENGTH - 1);
          break;
     case 1024:
          strncpy(message, IDS_PROTOCOL_1024, IDS_ERROR_MAX_LENGTH -1);
          break;
     case 1026:
          strncpy(message, IDS_PROTOCOL_1026, IDS_ERROR_MAX_LENGTH - 1);
          break;
     case 1031:
          strncpy(message, IDS_PROTOCOL_1031, IDS_ERROR_MAX_LENGTH - 1);
          break;
     case 1040:
          strncpy(message, IDS_PROTOCOL_1040, IDS_ERROR_MAX_LENGTH - 1);
          break; 
     case PROTERR_REQUEST_TIMED_OUT:
          strncpy(message, IDS_AUTHENTICATE_ERROR, IDS_ERROR_MAX_LENGTH - 1);
          break;
     case 1030:
          strncpy(message, IDS_PROTOCOL_1030, IDS_ERROR_MAX_LENGTH - 1);
          break; 
     case PROTERR_GENERAL_FAILURE:
     default:
          break;
     }

     if (connect_dialog)
     {
          gui_delete_connect();
     }
     
     m_OperationFailed = TRUE;
     
     if (strlen(message))
     {
          gui_create_error(message);
          gui_history_set_log_message(message);
     }
}

void gui_handle_timeout() 
{
     char errormessage[IDS_ERROR_MAX_LENGTH];
     strcpy(errormessage, "");
     
     /* TODO, if we had a reauthentication dialog open, it should be
      * destroyed here.
      */
       
     if (main_timer) {
          g_timer_destroy(main_timer);
          main_timer = NULL;
     }
     
     gui_history_set_log_message(IDS_CONN_CLOSED_MSG);
     
     gui_connect_button_set_text(" Connect ");
     m_Connected = FALSE;
     m_State = None;
     
     strncpy(errormessage, IDS_TIMEOUT_ERROR, IDS_ERROR_MAX_LENGTH -1);
     gui_create_error(errormessage);
}

void gui_notify(int message, int arg)
{
     char errormessage[IDS_ERROR_MAX_LENGTH];
     strcpy(errormessage, "");
     
     switch (message) {
     case NOT_AGENT_DISCOVERED:
          m_State = None;
          m_Connected = FALSE;
          m_Timeout = FALSE;
          m_OperationFailed = FALSE;
          
          gui_connect_button_enable();
          gui_update_status("Service detected");
          gui_connect_button_enable();
          
          gui_history_set_log_message(IDS_PAC_DISCOVERED_MSG);
          
          if (main_timer)
          {
               g_timer_destroy(main_timer);
               main_timer = NULL;
          }

          break;
     case NOT_NO_AGENT:
          gui_connect_button_disable();
          gui_update_status("Service not available");

          if (main_timer)
          {
               g_timer_destroy(main_timer);
               main_timer = NULL;
          }

          break;
     case NOT_CONNECTED:
          
          if (connect_dialog)
               gui_delete_connect();
          gui_update_status("Connected");
          gui_connect_button_enable();
          gui_connect_button_set_text(" Disconnect ");
          m_State = Connected;
          m_Connected = true;

          break;
     case NOT_DISCONNECTED:

          /* We are reusing connect_dialog as a disconnect
           * dialog.
           */
          if (connect_dialog)
               gui_delete_connect();

          if (main_timer)
          {
               g_timer_destroy(main_timer);
               main_timer = NULL;
          }

          gui_connect_button_disable();
          gui_connect_button_set_text(" Connect ");
          m_State = None;
          m_Connected = false;

          gui_history_set_log_message(IDS_CONN_CLOSED_MSG);

          break;
     case NOT_CONNECTION_STARTED:
          if (!main_timer)
          {
               gui_update_duration(0);
          }

          if (m_State != Reauthentication)
               m_State = Connecting;

          if (!connect_dialog)
               gui_create_connect();

          break;
     case NOT_OPERATION_PROGRESS:

          if (connect_dialog)
               gui_update_connect_progressbar((float)arg / 100);

          if (arg == 100) {
               if (!main_timer) {
                    main_timer = g_timer_new();
                    g_timer_start(main_timer);
               }

               if (!m_Connected)
                    gui_history_set_log_message(IDS_CONN_OPENED_MSG);
          }

          break;
     case NOT_DISCONNECTION_STARTED:
          m_State = Disconnecting;

          /* We are reusing connect_dialog as a disconnect
           * dialog.
           */
          if (!connect_dialog)
               gui_create_connect();

          break;
     case NOT_OPERATION_FAILED:
          switch (m_State) {
          case Connecting:
               /* OK to fall through */
          case Reauthentication:
               gui_handle_operation_failed(arg);

               if (m_Timeout)
               {
                    gui_handle_timeout();
               }
               break;
          case Connected:
               break;
          case Disconnecting:
               /* TODO, this might require a confirmation dialog.
                */
               break;
          default:
               break;
          }
          break;
     case NOT_CANCELLED:
          strncpy(errormessage, IDS_CANCELLED_INFO, IDS_ERROR_MAX_LENGTH - 1);
          gui_create_error(errormessage);

          break;
     case NOT_CONFIRM_RECONNECT:
          /* TODO, this might require a confirmation dialog.
           * 
           * gui_create_yesno("You need to reconnect to the service.\nDo you want to do it now?", gui_yesno_reauth_clicked, NULL, NULL);
           */
          break;
     case NOT_CONNECTION_TIMEOUT:
          switch (m_State) {
          case Disconnecting:
               /* OK to fall through */
          case Reauthentication:
               if (m_OperationFailed)
                    gui_handle_timeout();
               else
                    m_Timeout = TRUE;

               break;
          case Connected:
               /* OK to fall through */
          case None:
               gui_handle_timeout();
               break;
          default:
               break;
          }
          break;
     case NOT_UNSUPPORTED_VERSION:
          break;
     case NOT_PROTOCOL_RESTARTED:
          break;
     default:
          break;
     }
}

/* A refresh event for the connection duration counter.
 */
void gui_update_duration_event(struct Event *event)
{
     Event_handle gui_update_duration_handle;

     if (main_timer)
          gui_update_duration(g_timer_elapsed(main_timer, NULL));

     nc_add_to_events(gui_update_duration_event, 1000, NULL,
                      &gui_update_duration_handle);
}


/* A dummy event that makes sure our GUI gets refreshed.
 */
void gui_update_timer_event(struct Event *event)
{
     Event_handle gui_update_timer_handle;

     nc_add_to_events(gui_update_timer_event, 50, NULL, 
                      &gui_update_timer_handle);
}
