/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef AM_GUI_H
#define AM_GUI_H

#include <naap_event.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * State of the authentication
 */
typedef enum {
        None,
        Connecting,
        Connected,
        Disconnecting,
        Reauthentication
} AuthenticationState;
     
#define NOT_AGENT_DISCOVERED       0x0001
#define NOT_CONNECTION_STARTED     0x0002
#define NOT_CONNECTED              0x0003
#define NOT_DISCONNECTION_STARTED  0x0004
#define NOT_DISCONNECTED           0x0005
#define NOT_CANCELLED              0x0006
#define NOT_OPERATION_FAILED       0x0007
#define NOT_OPERATION_PROGRESS     0x0008
#define NOT_CONFIRM_RECONNECT      0x0009
#define NOT_CONNECTION_TIMEOUT     0x000A
#define NOT_NO_AGENT               0x000B
#define NOT_UNSUPPORTED_VERSION    0x000C
#define NOT_PROTOCOL_RESTARTED     0x000D

#define PROTOK                     0x1111
#define PROTERR_GENERAL_FAILURE    0x0000
#define PROTERR_SIM_FAILURE        0x0800
#define PROTERR_REQUEST_TIMED_OUT  0x0801

#define IDS_ERROR_MAX_LENGTH 200

#define IDS_SIM_ERROR "SIM card failure"
#define IDS_PROTOCOL_1024 "This service provider does not have a roaming agreement with your service provider. (1024)"
#define IDS_PROTOCOL_1026 "Call barring is activated. (1026)"
#define IDS_PROTOCOL_1030 "Connection failed. (1030)"
#define IDS_PROTOCOL_1031 "You have not subscribed to this service. Please contact your service provider. (1031)"
#define IDS_PROTOCOL_1040 "Network failure. (1040)"
#define IDS_AUTHENTICATE_ERROR "Unable to connect to the service. (501)"
#define IDS_PROTOCOL_GENERAL "Connection failed. (502)"

#define IDS_ERROR_CAPTION "SIM services"
#define IDS_INFO_CAPTION  "SIM services"
#define IDS_CANCELLED_INFO "Connection cancelled"
#define IDS_TIMEOUT_ERROR "Connection timed out."

#define IDS_CONN_CLOSED_MSG "Connection closed"
#define IDS_CONN_OPENED_MSG "Connection opened"
#define IDS_PAC_DISCOVERED_MSG "Service detected"

void gui_update_connect_progressbar(float percentage);
void gui_create_history(void);
void gui_create_connect(void);
void gui_delete_connect(void);
void gui_create_main(void);
void gui_update_timer_event(struct Event *event);
void gui_update_duration_event(struct Event *event);
void gui_create_error(char *error);
void gui_update_status(char *statustext);
void gui_connect_button_enable(void);
void gui_connect_button_disable(void);
void gui_connect_button_set_text(char *text);
void gui_notify(int message, int arg);
void gui_handle_timeout();

#ifdef __cplusplus
}
#endif

#endif /* ifndef AM_GUI_H */
     
