/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <unistd.h>
#include <string.h>
#include "naap_am.h"
#include "am_init.h"
#include "naap_event.h"
#include "am_fileconfig.h"
#include "naap_socket.h"
#include "am_global.h"
#include "am_socket.h"
#include "naap_ui.h"
#include "naap_dt.h"
#include <gtk/gtk.h>
#include <gtk/gtkmain.h>
#include "am_gui.h"

/* This variable tells whether we are running as a client or an agent */
NAAP_entity am_naap_mode = UNINITIALIZED;

/* This variable contains our own IP address */
IP_addr am_own_ip_addr = IP_ADDR_ANY;

/* This variable contains our realm */
char am_realm[AM_MAX_REALM_LENGTH + 1];

/* Name of the logfile */
char* debuglogfile = NULL;

int number_of_rands = 1;

int stop_flag = 0;

int main(int argc, char** argv)
{
  static Socket_handle* socket_table;
  struct timeval* timeout;
  fd_set fds;
  int selected;
  int fd; 
  int max_fd;
  static int socket_table_size;
  int i;
  struct ifconf ifc;
  struct ifreq ifrs[100];
  int sk;
  
  am_init();

  /* We try to find out the address of our interface. 
   */
  sk = socket(PF_INET, SOCK_DGRAM, 0);
  
  ifc.ifc_len = sizeof(ifrs);
  ifc.ifc_req = ifrs;
  ioctl(sk, SIOCGIFCONF, &ifc);
  
  for (i = 0; i < ifc.ifc_len / sizeof(struct ifreq); i++) {
       struct ifreq *ifr = &ifrs[i];

       /* TODO, is this the best way to do this? */
       if (!strcmp(ifr->ifr_name, "eth0")) {
            am_own_ip_addr = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr.s_addr;
       }
  }
  close(sk);

#ifndef RELEASE_CODE
  if(argc == 3)
  {
       debuglogfile = argv[1]; /* This line must be before fileconfig_core()
                                  because fileconfiguration utilities might
                                  try to write to debuglogfile. */
       fileconfig_core(argv[2]);
  }
  else if (argc == 2) {
       debuglogfile = argv[1]; 

       fileconfig_core(NULL);
  } else {
       printf("Usage: %s logfile <configurationfile>\n", argv[0]);
       am_cleanup();
       return EXIT_FAILURE;
  }
#else
  if(argc == 2)
  {
       fileconfig_core(argv[1]);
  } else if (argc == 1) {
       fileconfig_core(NULL);
  } else {
       printf("Usage: %s <configurationfile>\n", argv[0]);
       am_cleanup();
       return EXIT_FAILURE;
  }
#endif /* #ifndef RELEASE_CODE */
  
  if(nc_handle_init(am_naap_mode, am_own_ip_addr) > 0)
  {
       DT((DT_ERROR, "ERROR: main: nc_handle_init failed, exiting\n"));
       am_cleanup();
       return EXIT_FAILURE;
  }

  gtk_set_locale();
  gtk_init(&argc, &argv);
  gui_create_main();
  gui_create_history();

#ifndef RELEASE_CODE
  printf("c - Connect\nx - Cancel\nd - Disconnect\nq - Quit\n");
#endif /* #ifndef RELEASE_CODE */

  while(!stop_flag)
  {		
       while (gtk_events_pending())
            gtk_main_iteration();
       
       socket_table = nc_get_receiving_sockets(&socket_table_size);
       
       /* Convert socket table to fd_set */
       FD_ZERO(&fds);
       
       fd = max_fd = 0;
       
       for(i = 0; i < socket_table_size; i++)
       {
            fd = get_socket_descriptor(socket_table[i]);
            if (fd > max_fd)
	    {
                 max_fd = fd;
	    }
            FD_SET(fd, &fds);
       }

#ifndef RELEASE_CODE
       /* Add standard input to fd_set fds */
       fd = 0;
       FD_SET(fd, &fds);
#endif /* #ifndef RELEASE_CODE */

       timeout = nc_time_to_next_event();

       selected = select(max_fd+1, &fds, NULL, NULL, timeout);
       
       if(selected < 0)
            continue;
       
       if(selected  > 0)
       {
	  for(i = 0; i < socket_table_size; i++)
          { 
               if(FD_ISSET(get_socket_descriptor(socket_table[i]), &fds))
               {
                    nc_handle_socket(socket_table[i]);
               }	
          }
          
#ifndef RELEASE_CODE
	  if(FD_ISSET(0, &fds))
          {
               char c;
               char tmp;
               
               /* Read one character */
               c = tmp = fgetc(stdin);
               
               /* Read other possibly typed characters (which are ignored) */
               while((tmp != '\n') && (tmp != EOF))
               {
                    tmp = fgetc(stdin);
               }
               
               switch (c)
               {
               case 'c':
               case 'C':
                    nc_connect_clicked();
                    break;
               case 'x':
               case 'X':
                    nc_cancel_clicked();
                    break;
               case 'd':
               case 'D':
                    nc_disconnect_clicked();
                    break;
               case 'q':
               case 'Q':
               case EOF:
                    stop_flag = 1;
                    break;
               default:
                    break;
               }
          }
#endif /* #ifndef RELEASE_CODE */
       }
       nc_handle_events();
  }

  DT((DT_AM, "main: Calling nc_handle_cleanup()\n"));
  nc_handle_cleanup();
  
  DT((DT_AM, "main: Calling am_cleanup()\n"));
  am_cleanup();
  
  DT((DT_AM, "main: Exiting\n"));
  return 0;
}
