/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include "naap_am.h"
#include "naap_notif.h"
#include "naap_dt.h"
#include "am_global.h"
#include "naap_const.h"
#include "am_gui.h"

static bool m_pendingCancel;

static void reset()
{
     m_pendingCancel = false;
}

void am_not_discovering_agent()
{
     DT((DT_AM, "am_not_discovering_agent()\n"));
     
     if(m_pendingCancel)
     {
          gui_notify(NOT_CANCELLED, PROTOK);
          reset();
     }

     /* Core has destroyed all events.
      */
     gui_update_timer_event(NULL);
     gui_update_duration_event(NULL);
}

void am_not_no_agent()
{
     DT((DT_AM, "am_not_no_agent()\n"));
     
     gui_notify(NOT_NO_AGENT, PROTOK);
}

void am_not_agent_discovered()
{
     DT((DT_AM, "am_not_agent_discovered()\n"));
     
     gui_notify(NOT_AGENT_DISCOVERED, PROTOK);
}

void am_not_getting_imsi()
{
     DT((DT_AM, "am_not_getting_imsi()\n"));
     
     gui_notify(NOT_CONNECTION_STARTED, PROTOK);
}

void am_not_sim_failure()
{
     DT((DT_AM, "am_not_sim_failure()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_SIM_FAILURE);
     reset();
}

void am_not_waiting_for_start()
{
     DT((DT_AM, "am_not_waiting_for_start()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 20);
}

void am_not_failure(u_int8_t* reply_message, u_int16_t reply_message_len)
{
     int failureCode = 0;
     char msg[16];

     DT((DT_AM, "am_not_failure()\n"));
     
     if(reply_message_len > sizeof(msg))
          reply_message_len = sizeof(msg); /* truncate long strings */
     
     /* make sure the string is null-terminated */
     memcpy(msg,reply_message,reply_message_len);
     msg[reply_message_len-1]='\0';
     
     /* parse failure code from ASCII string */
     failureCode=atoi(msg);
     
     gui_notify(NOT_OPERATION_FAILED, failureCode);
     reset();
}

void am_not_unsupported_version()
{
     DT((DT_AM, "am_not_unsupported_version()\n"));
     
     gui_notify(NOT_UNSUPPORTED_VERSION, PROTOK);
     reset();
}

void am_not_start_timed_out()
{
     DT((DT_AM, "am_not_start_timed_out()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_REQUEST_TIMED_OUT);
     reset();
}

void am_not_waiting_for_challenge()
{
     DT((DT_AM, "am_not_waiting_for_challenge()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 40);
}

void am_not_challenge_timed_out()
{
     DT((DT_AM, "am_not_challenge_timed_out()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_REQUEST_TIMED_OUT);
     reset();
}

void am_not_getting_kc_sres()
{
     DT((DT_AM, "am_not_getting_kc_sres()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 60);
}

void am_not_mac_rand_failed()
{
     DT((DT_AM, "am_not_mac_rand_failed()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_GENERAL_FAILURE);
     reset();
}

void am_not_waiting_for_success()
{
     DT((DT_AM, "am_not_waiting_for_success()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 80);
}

void am_not_success_timed_out()
{
     DT((DT_AM, "am_not_success_timed_out()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_REQUEST_TIMED_OUT);
     reset();
}

void am_not_authentication_successful()
{
     DT((DT_AM, "am_not_authentication_successful()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 100);
     gui_notify(NOT_CONNECTED, PROTOK);
     reset();
}

void am_not_connected()
{
     DT((DT_AM, "am_not_connected()\n"));
     
     if(m_pendingCancel)
     {
          gui_notify(NOT_CANCELLED, PROTOK);
          reset();
     }
}

void am_not_synchronizing_failed()
{
     DT((DT_AM, "am_not_synchronizing_failed()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_GENERAL_FAILURE);
     reset();
}

void am_not_naap_registration_successful()
{
     DT((DT_AM, "am_not_naap_registration_successful()\n"));
     
     /* No need to do anything */
}

void am_not_disconnect_successful()
{
     DT((DT_AM, "am_not_disconnect_successful()\n"));
     
     gui_notify(NOT_OPERATION_PROGRESS, 100);
     gui_notify(NOT_DISCONNECTED, PROTOK);
     reset();
}

void am_not_naap_registration_expired()
{
     DT((DT_AM, "am_not_naap_registration_expired()\n"));
     
     gui_notify(NOT_CONNECTION_TIMEOUT, PROTOK);
     reset();
}

void am_not_disconnecting()
{
     DT((DT_AM, "am_not_disconnecting()\n"));
     
     gui_notify(NOT_DISCONNECTION_STARTED, PROTOK);
     gui_notify(NOT_OPERATION_PROGRESS, 40);
     reset();
}

void am_not_disconnect_failed()
{
     DT((DT_AM, "am_disconnect_failed()\n"));
     
     gui_notify(NOT_OPERATION_FAILED, PROTERR_REQUEST_TIMED_OUT);
     reset();
}

void am_not_protocol_restarted()
{
     DT((DT_AM, "am_protocol_restarted()\n"));
     
     gui_notify(NOT_PROTOCOL_RESTARTED, PROTOK);
     reset();
}

void am_not_reply_with_invalid_authenticator()
{
     DT((DT_AM, "am_not_reply_with_invalid_authenticator()\n"));
     
     /* No reason to do anything */
     
     return;
}
