/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <string.h>
#include <stdio.h>
#include "naap_am.h"
#include "naap_sim.h"
#include "naap_event.h"
#include "naap_memory.h"
#include "naap_dt.h"
#include "naap_const.h"
#include "am_sim_d211.h"
#include "am_global.h"

/* buffer size for IPC messages */
#define BUF_SIZE 3000

/* mask for getting msq key from MessageID of the IPC message */
#ifndef MSQ_ID_MASK
#define MSQ_ID_MASK 0x03FF0000
#endif

int rsp_msqid = -1;
static int seed = 1;

int send_message(PIPCMESSAGE pIpc, PIPCRSP pIpc_rsp){
     int TID = 0;
     int ret = 0;
     int req_key;
     int req_msqid;
     int msg_len;
     int rsp_key;
     MSQMSG* pmsg;
     
     /* sanity check */
     if (!pIpc || !pIpc_rsp){
          DT((DT_ERROR, "null arguments, pIpc: %p, pIpc_rsp: %p\n", 
              pIpc, pIpc_rsp));
          return -1;
     }        
     
     msg_len = pIpc->Header.Len; 

     /* key for responce msg queue */
     rsp_key = KEY_OF_MSQ(pIpc->Header.MessageID | IPC_RSP_BASE);
     
     /* get rsp msg queue
      * TODO, permissions all
      */
     rsp_msqid = msgget(rsp_key, 0666 | IPC_CREAT); 
     if (rsp_msqid == -1){
          DT((DT_ERROR, "msgget failed for responce queue\n"));
          return -1;
     }        
     
     if (pIpc->Header.Timeout == 0)
          pIpc->Header.Timeout = 5000;
     
     /* key for msg queue */
     req_key = KEY_OF_MSQ(pIpc->Header.MessageID);

     /* TODO, permissions all */
     req_msqid = msgget(req_key, 0666);
     
     if (req_msqid == -1){
          DT((DT_ERROR, "msgget failed for request queue, msq: %li\n", 
              pIpc->Header.MessageID & MSQ_ID_MASK));
          return -1;
     }
     TID = pIpc->Header.TID;

     pmsg = (MSQMSG*)malloc(msg_len + sizeof(long int));
     
     if(!pmsg){
          DT((DT_ERROR, "malloc\n"));
          return -1;
     }
     memset(pmsg, 0, msg_len + sizeof(long int));
     pmsg->mtype = REQ_MSG_TYPE(pIpc->Header.MessageID);
     memcpy(pmsg->ipc_msg, pIpc, msg_len);
     
     DT((DT_AM, "sending message to msg queue %i, key 0x%08lx\n", 
         req_msqid, req_key));
     DT((DT_AM, "send request: \n"));
     DT((DT_AM, "TID: %x\n", pIpc->Header.TID));
     DT((DT_AM, "Message ID: %x\n", pIpc->Header.MessageID));
     DT((DT_AM, "Timeout: %li\n", pIpc->Header.Timeout));
     DT((DT_AM, "Len: %li\n\n", pIpc->Header.Len));
     
     DT((DT_AM, "waiting responce message from msg queue %i, key 0x%08lx\n",
         rsp_msqid, rsp_key));

     if (msgsnd(req_msqid, (const void*)pmsg, msg_len, IPC_NOWAIT) == -1) {
               DT((DT_ERROR, "msgsnd failed, msq: %i, key: 0x%08lx, len: %i\n", 
              req_msqid, req_key, msg_len - sizeof(long int)));
          ret = -1; 
     }
     else if ( (ret = msgrcv(rsp_msqid, (void*)pIpc_rsp, BUF_SIZE - sizeof(long int), 
                             RSP_MSG_TYPE((pIpc->Header.MessageID | IPC_RSP_BASE), 
                                          pIpc->Header.TID), 0)) == -1) {
          DT((DT_ERROR, "msgrcv failed, msq: 0x%x\n", rsp_msqid));
          ret = -1; 
     }
     else { /* request succeeded */
          char* tmp;
          int len = 0, i = 0;
          memmove(pIpc_rsp, ((MSQMSG*)pIpc_rsp)->ipc_msg, ret);

          DT((DT_AM, "received responce: \n"));
          DT((DT_AM, "TID: %x\n", pIpc_rsp->a.b.Header.TID));
          DT((DT_AM, "Message ID: %x\n", pIpc_rsp->a.b.Header.MessageID));
          DT((DT_AM, "Timeout: %li\n", pIpc_rsp->a.b.Header.Timeout));
          DT((DT_AM, "Len: %li\n", pIpc_rsp->a.b.Header.Len));
          DT((DT_AM, "Status: "));
          if (pIpc_rsp->a.b.Status != 0) {
               DT((DT_ERROR, "ERROR %li\n", pIpc_rsp->a.b.Status));
               switch(pIpc_rsp->a.b.Status){
               case IPC_STATUS_ERROR:
                    DT((DT_ERROR, "An error occurred\n"));
                    break;
               case IPC_STATUS_NOT_HANDLED:
                    DT((DT_ERROR, "Command was not handled\n"));
                    break;
               case IPC_STATUS_INVALIDPARAM:
                    DT((DT_ERROR, "Invalid parameters\n"));
                    break;
               case IPC_STATUS_FATAL_ERROR:
                    DT((DT_ERROR, "A fatal error occurred\n"));
                    break;
               case IPC_STATUS_PIPE_TIMEOUT:
               case IPC_STATUS_ACTION_TIMEOUT:
               case IPC_STATUS_REQUEST_TIMEOUT:
                    DT((DT_ERROR, "Command timed out\n"));
                    break;
               case IPC_STATUS_NOT_IMPLEMENTED:
                    DT((DT_ERROR, "Not implemented\n"));
                    break;
               default:
                    DT((DT_ERROR, "Unknown error occurred\n"));
                    break;
               }
          } else {
               DT((DT_AM, "%li\n", pIpc_rsp->a.b.Status));
          }
          
          len = ret - sizeof(IPCRSPHEADER);
          DT((DT_AM, "DataLen %i\n", len));
          tmp = (char*)pIpc_rsp->Data;
          for (i = 0; i < len; i++){
               DT((DT_AM, "%02x ", 0x000000ff&tmp[i]));
          }
          DT((DT_AM, "\n"));
          
     }

     free(pmsg);

#if 0
     if (rsp_msqid != -1){
          if (msgctl(rsp_msqid, IPC_RMID, 0) == -1){
               DT((DT_ERROR, "msgctl IPC_RMID failed\n"));
               ret = -1;
          }
          rsp_msqid = -1;
     }
#endif /* #if 0 */
     
     return ret;
}

/* Read a random number into a buffer */
char* read_random (char* buf, int length)
{
        FILE *f = NULL;
        
        if (!buf)
             return NULL;
        
        f = fopen("/dev/random","rb");
        if (!f) {
             DT((DT_ERROR, "Could not open /dev/random\n"));
             return NULL;
        }
        if (!fgets(buf,length,f)) {
             DT((DT_ERROR, "Could not read /dev/random\n"));
             return NULL;
        }       
        fclose(f);
        
        return buf;
}

int get_next_tid()
{
     int SEQ_TID;
     unsigned char x[3];

     /* Use /dev/random only for the first seed */
     if (seed == 1) {
          if (!read_random(x, 3)) {
               return -1;
          }
          seed=*(int*)x;
     }
     
     srand(++seed);
        
     /* SEQ_TID cannot be zero, this is very very rare, but... */
     do {
          SEQ_TID = rand();
     } while (SEQ_TID == 0); 

     return SEQ_TID;
}

int32_t am_req_imsi()
{	
     char buffer[BUF_SIZE];
     PIPCMESSAGE pIpc = (PIPCMESSAGE) buffer;
     PIPCRSP pIpc_rsp = (PIPCRSP) buffer;
     PREQSIMIMSIREAD pIpcIR = (PREQSIMIMSIREAD)pIpc;
     char IMSI[NAAP_MAX_IMSI_LENGTH];
     char IMSItmp[NAAP_MAX_IMSI_LENGTH * 2];
     char *imsi;
     int len, ret;
     char my_realm[AM_MAX_REALM_LENGTH];
     char mnc[4], mcc[4];

     bzero(buffer,sizeof(buffer));

     pIpcIR->Header.TID = get_next_tid();
     pIpcIR->Header.MessageID = REQ_SIM_READIMSI;
     pIpcIR->Header.Timeout = 0;
     pIpcIR->Header.Len = sizeof(REQSIMIMSIREAD);
     
     /* Synchronic love */
     if ((ret = send_message(pIpc, pIpc_rsp)) < 0) {
          DT((DT_ERROR, "Error: send_message failed in am_req_imsi\n"));
          return -1;
     }

     /* An error has occured! */
     if (pIpc_rsp->a.b.Status != 0) {
          return -1;
     }

     ret += sizeof(long int);
     len = ret - sizeof(IPCRSPHEADER);
     imsi = (char *)pIpc_rsp->Data+1;

     IMSI[0] = ((imsi[0] >> 4) << 4) | (imsi[1] & 0x0F);
     IMSI[1] = ((imsi[1] >> 4) << 4) | (imsi[2] & 0x0F);
     IMSI[2] = ((imsi[2] >> 4) << 4) | (imsi[3] & 0x0F);
     IMSI[3] = ((imsi[3] >> 4) << 4) | (imsi[4] & 0x0F);
     IMSI[4] = ((imsi[4] >> 4) << 4) | (imsi[5] & 0x0F);
     IMSI[5] = ((imsi[5] >> 4) << 4) | (imsi[6] & 0x0F);
     IMSI[6] = ((imsi[6] >> 4) << 4) | (imsi[7] & 0x0F);
     IMSI[7] = ((imsi[7] >> 4)) & 0x0F;

     sprintf(IMSItmp, "%02X%02X%02X%02X%02X%02X%02X%X", IMSI[0], IMSI[1],
             IMSI[2], IMSI[3], IMSI[4], IMSI[5], IMSI[6], IMSI[7]);

     /* Check if our realm is owlan.org and change it
      * accordingly.
      */
     if (!strcmp(am_realm, "owlan.org")) {
          strncpy(mnc, IMSItmp+3, 3);
          strncpy(mcc, IMSItmp, 3);
          sprintf(my_realm, "mnc%s.mcc%s.owlan.org", mnc, mcc); 
     } else
          strncpy(my_realm, am_realm, AM_MAX_REALM_LENGTH - 1);

     DT((DT_AM, "IMSI is %s.\n", IMSItmp));

     DT((DT_AM, "Realm is %s.\n", my_realm));

     nc_ans_imsi(IMSItmp, strlen(IMSItmp), my_realm, strlen(my_realm));

     return 0;
}

int32_t am_req_kc_sres(u_int32_t context, u_int8_t* rand, u_int16_t rand_len)
{
     char buffer[BUF_SIZE];
     PIPCMESSAGE pIpc = (PIPCMESSAGE) buffer;
     PIPCRSP pIpc_rsp = (PIPCRSP) buffer;
     PREQSIMRUNGSMALG pIpcRG = (PREQSIMRUNGSMALG)pIpc;
     char kc[NAAP_KC_LENGTH], sres[NAAP_SRES_LENGTH];

     bzero(buffer,sizeof(buffer));

     pIpcRG->Header.TID = get_next_tid();
     pIpcRG->Header.MessageID = REQ_SIM_RUNGSMALG;
     pIpcRG->Header.Timeout = 0;
     pIpcRG->Header.Len = sizeof(REQSIMRUNGSMALG);
     pIpcRG->Type = GSM_ALGORITHM;
     memcpy(pIpcRG->Random, rand, rand_len);

     /* Synchronic love */
     if ((send_message(pIpc, pIpc_rsp)) < 0) {
          DT((DT_ERROR, "Error: send_message failed in am_req_kc_sres\n"));
          return -1;
     }

     /* An error has occured! */
     if (pIpc_rsp->a.b.Status != 0) {
          return -1;
     }

     memcpy(sres, (char *)pIpc_rsp->Data, NAAP_SRES_LENGTH);
     memcpy(kc, (char *)pIpc_rsp->Data+NAAP_SRES_LENGTH, NAAP_KC_LENGTH);

     nc_ans_kc_sres(context, kc, NAAP_KC_LENGTH, sres, NAAP_SRES_LENGTH);

     return 0;
}
