/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef AM_SIM_D211_H
#define AM_SIM_D211_H

#ifdef __cplusplus
extern "C" {
#endif

/* The following code has been taken directly from D211 Linux driver sources.
 *
 * Anonymous unions and structs used in e.g. struct IPCRSP don't seem to compile
 * on regular gcc versions (2.95.4 on Debian/Sid was tested). They do, however, 
 * compile on Red Hat's version of gcc (2.96 on Red Hat 7.2). I have therefore
 * modified the necessary unions and structs as a stop-gap measure. In the long
 * run the original (Windows) code should be changed so that the code below could
 * be #included from D211 sources without copying.
 *    -- tm
 */

#ifndef __KERNEL__
#include <string.h>
#include <time.h>
#include <assert.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>

extern int errno;
#define CHR_DEV "/dev/nokia_cs"

#endif //__KERNEL__


#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE  1
#endif

/* empty defines, not needed in Linux */
//TODO?
#define IN
#define OUT
#define OPTIONAL
#define UNALIGNED
#define CALLBACK
#define __stdcall
#define  D211IPCAPI
#define WINAPI
#define __declspec(dllimport)
#define APIENTRY

/* porting defines */
//TODO?
#define TEXT(x) x
#define _T(x) x
#define INFINITE 0xffffffff
#define INVALID_HANDLE_VALUE 0

/* function defines */
#define _tcsdup strdup
#define _tcslen strlen
#define wcslen strlen
#define _tcschr strchr 
#define _tcscpy strcpy
#define wcscpy strcpy
#define _tcsncpy strncpy

#define _tfopen fopen 
#define _ftprintf fprintf
#define strnicmp strncasecmp
//TODO?
#define __try try
#define __except(a) catch(...)
#define _ASSERT(a) assert(a!=0)


//TODO section
#define InternetDial(a,b,c,d,e) ERROR_NOT_READY
#define InternetHangUp(a,b) ERROR_NOT_READY
#define CloseHandleEvent(x) do { DeleteEvent(x); x=0; } while(0)  //TODO?
#define CloseHandleNull(x)     //TODO?
#define CloseHandleZero(x) x=0     //TODO?
#define GetOverlappedResult(a,b,c,d) 0
#define TerminateThread(a,b) 

#define SetThreadDesktop(d)
#define OutputDebugString(t) printf("Debug: %s\n", t);
#define TlsSetValue(a,b) printf((char*) b)
#define MessageBox(a, text, type, error) do { printf("MessageBox " type "(%i):\n %s \n", error, text); return 0; } while (0)
#define GetTickCount() time(0)
#define SetEvent(a) SetEventState(a, 1)
#define GetCurrentProcessId() 0
#define GetCurrentThreadId() 0
#define UnmapViewOfFile(shm_view_of_server)
//#define CloseHandle(a)
#define MapViewOfFile(handle, flags, a, b, c) NULL
#define _msize(b) 0
#define EnterCriticalSection(phandle)
#define LeaveCriticalSection(phandle)
#define InitializeCriticalSection(phandle)
#define DeleteCriticalSection(phandle)
#define CreateMutex(psa, init, sem_name_or_null) 0
#define ReleaseMutex(handle)
#define Sleep(time) sleep(time)
        //TODO: should be atomic?
#define InterlockedExchange(pvar,val) *pvar=val
        
#define DLL_PROCESS_ATTACH 4
#define DLL_THREAD_ATTACH 5
#define DLL_THREAD_DETACH 6
#define DLL_PROCESS_DETACH 7

//TODO?
#define ERROR_SUCCESS         0
#define ERROR_ALREADY_EXISTS  1
#define ERROR_READ_FAULT      2
#define ERROR_NOT_READY       3
#define ERROR_IO_INCOMPLETE   4
#define ERROR_IO_PENDING      5

#define FILE_MAP_ALL_ACCESS   10

#define LOBYTE(a) (BYTE)(0x00ff & a)
#define HIBYTE(a) (BYTE)((0xff00 & a) >> 8)

#define HIWORD(l)              ((WORD)((DWORD)(l) >> 16))


#define GlobalAllocPtr(flags,size) (HANDLE)malloc(size)
#define GlobalFreePtr(handle) free(handle)

#define MB_ICONWARNING 1

#define WAIT_TIMEOUT  0
#define WAIT_OBJECT_0 1
#define WAIT_FAILED   1
#define WAIT_ABANDONED_0 10
//till here

/* Standard data types */
typedef const void                  *PCVOID,   *LPCVOID;
typedef int             BOOL,       *PBOOL,    *LPBOOL;
typedef unsigned char   BYTE,       *PBYTE,    *LPBYTE;
typedef unsigned char   UCHAR,      *PUCHAR;
typedef unsigned short  USHORT,     *PUSHORT,  *LPUSHORT;
typedef unsigned short  WORD,       *PWORD,    *LPWORD;
#ifndef __KERNEL__  //TODO, INT and UINT also in types.h
typedef int             INT,        *PINT,     *LPINT;
typedef unsigned int    UINT,       *PUINT,    *LPUINT;
#endif
typedef unsigned long   DWORD,      *PDWORD,   *LPDWORD;
typedef unsigned long   ULONG,      *PULONG,   *LPULONG;
typedef float           FLOAT,      *PFLOAT,   *LPFLOAT;
typedef double          DOUBLE,     *PDOUBLE,  *LPDOUBLE;

/* Define the basic types */
#ifndef VOID
#define VOID void
#endif
typedef VOID           *PVOID,      *LPVOID;
typedef BYTE            BOOLEAN,    *PBOOLEAN;
typedef char            CHAR,       *PCHAR;
typedef short           SHORT,      *PSHORT;
typedef long            LONG,       *PLONG,    *LPLONG;

/* ANSI string types */
typedef CHAR           *PCH,        *LPCH;
typedef const CHAR     *PCCH,       *LPCCH;
typedef CHAR           *PSTR,       *LPSTR;
typedef const CHAR     *PCSTR,      *LPCSTR;

/* Unicode string types */
typedef CHAR          *PWCH,       *LPWCH;
typedef const CHAR    *PCWCH,      *LPCWCH;
typedef CHAR          *PWSTR,      *LPWSTR;
typedef const CHAR    *PCWSTR,     *LPCWSTR;

typedef CHAR           TCHAR,      *PTCHAR, WCHAR;
typedef LPSTR          PTSTR,       LPTSTR;
typedef LPCSTR         PCTSTR,      LPCTSTR;

/* Misc common WIN32 types */
typedef LONG            HRESULT;
typedef DWORD           LCID,       *PLCID;
typedef WORD            LANGID;
typedef DWORD           EXECUTION_STATE;

//TODO?
typedef unsigned long long __uint64;
typedef __uint64        ULONGLONG,  *PULONGLONG;
typedef ULONGLONG       DWORDLONG,  *PDWORDLONG;
typedef unsigned long HMODULE;
//typedef unsigned long IPCSTATUS;
typedef unsigned int HANDLE, *LPHANDLE, CRITICAL_SECTION, HWND, HDESK;
typedef DWORD (*LPTHREAD_START_ROUTINE)(LPVOID);
typedef void* (*VTHREAD_START_ROUTINE)(LPVOID);
//typedef HANDLE (*EVENT_FUNC)(HANDLE);

typedef unsigned long SYSTEMTIME, FILETIME, HKEY; 

//TODO section
/* The 'overlapped' data structure used by async I/O functions.
 */
typedef struct {
        DWORD Internal;
        DWORD InternalHigh;
        DWORD Offset;
        DWORD OffsetHigh;
        HANDLE hEvent;
} OVERLAPPED, *LPOVERLAPPED;

/*
 * The exception record used by Win32 to give additional information 
 * about exception to exception handlers.
 */

#define EXCEPTION_MAXIMUM_PARAMETERS 15

typedef struct __EXCEPTION_RECORD
{
    DWORD    ExceptionCode;
    DWORD    ExceptionFlags;
    struct __EXCEPTION_RECORD *ExceptionRecord;

    LPVOID   ExceptionAddress;
    DWORD    NumberParameters;
    DWORD    ExceptionInformation[EXCEPTION_MAXIMUM_PARAMETERS];
} EXCEPTION_RECORD, *PEXCEPTION_RECORD;

typedef struct _CONTEXT
{
    /* selected by CONTEXT_FLOATING_POINT */
    ULONGLONG  FltF0;
    ULONGLONG FltF1;
    ULONGLONG FltF2;
    ULONGLONG FltF3;
    ULONGLONG FltF4;
    ULONGLONG FltF5;
    ULONGLONG FltF6;
    ULONGLONG FltF7;
    ULONGLONG FltF8;
    ULONGLONG FltF9;
    ULONGLONG FltF10;
    ULONGLONG FltF11;
    ULONGLONG FltF12;
    ULONGLONG FltF13;
    ULONGLONG FltF14;
    ULONGLONG FltF15;
    ULONGLONG FltF16;
    ULONGLONG FltF17;
    ULONGLONG FltF18;
    ULONGLONG FltF19;
    ULONGLONG FltF20;
    ULONGLONG FltF21;
    ULONGLONG FltF22;
    ULONGLONG FltF23;
    ULONGLONG FltF24;
    ULONGLONG FltF25;
    ULONGLONG FltF26;
    ULONGLONG FltF27;
    ULONGLONG FltF28;
    ULONGLONG FltF29;
    ULONGLONG FltF30;
    ULONGLONG FltF31;

    /* selected by CONTEXT_INTEGER */
    ULONGLONG IntV0;
    ULONGLONG IntT0;
    ULONGLONG IntT1;
    ULONGLONG IntT2;
    ULONGLONG IntT3;
    ULONGLONG IntT4;
    ULONGLONG IntT5;
    ULONGLONG IntT6;
    ULONGLONG IntT7;
    ULONGLONG IntS0;
    ULONGLONG IntS1;
    ULONGLONG IntS2;
    ULONGLONG IntS3;
    ULONGLONG IntS4;
    ULONGLONG IntS5;
    ULONGLONG IntFp;
    ULONGLONG IntA0;
    ULONGLONG IntA1;
    ULONGLONG IntA2;
    ULONGLONG IntA3;
    ULONGLONG IntA4;
    ULONGLONG IntA5;
    ULONGLONG IntT8;
    ULONGLONG IntT9;
    ULONGLONG IntT10;
    ULONGLONG IntT11;
    ULONGLONG IntRa;
    ULONGLONG IntT12;
    ULONGLONG IntAt;
    ULONGLONG IntGp;
    ULONGLONG IntSp;
    ULONGLONG IntZero;

    /* selected by CONTEXT_FLOATING_POINT */
    ULONGLONG Fpcr;
    ULONGLONG SoftFpcr;

    /* selected by CONTEXT_CONTROL */
    ULONGLONG Fir;
    DWORD Psr;
    DWORD ContextFlags;
    DWORD Fill[4];
} CONTEXT, *PCONTEXT;
/*
 * The exception pointers structure passed to exception filters
 * in except() and the UnhandledExceptionFilter().
 */
 
typedef struct _EXCEPTION_POINTERS 
{
  PEXCEPTION_RECORD  ExceptionRecord;
  PCONTEXT           ContextRecord;
} EXCEPTION_POINTERS, *PEXCEPTION_POINTERS, *LPEXCEPTION_POINTERS;

typedef struct {
        DWORD dwOSVersionInfoSize;
        DWORD dwMajorVersion;
        DWORD dwMinorVersion;
        DWORD dwBuildNumber;
        DWORD dwPlatformId;
        CHAR szCSDVersion[128];
} OSVERSIONINFO, OSVERSIONINFOA, *POSVERSIONINFOA, *LPOSVERSIONINFOA;

/*********************
 * Group definitions *
 *********************/

// IPC related
#define IPC_IPC_BASE		  (1<<16)

// MIB group is used for status codes from WLANMIB and CTLMIB
#define IPC_MIB_BASE          (2<<16)

// Control related
#define IPC_CTL_BASE          (10<<16)
#define IPC_CTLMIB_BASE       (11<<16)
#define IPC_PROF_BASE         (12<<16)
#define IPC_PWEP_BASE         (13<<16)
#define IPC_TRAFFIC_BASE      (14<<16)
#define IPC_HISTORY_BASE      (15<<16)
#define IPC_SCFS_BASE         (16<<16)

// PopUI related
#define IPC_POPUI_BASE        (20<<16)

// WLAN related
#define IPC_WLANCTL_BASE      (30<<16)
#define IPC_WLANMIB_BASE      (31<<16)
#define IPC_WLAN_BASE         (32<<16)
#define IPC_WLANTEST_BASE     (33<<16)

// GSM related
#define IPC_PHONE_BASE        (40<<16)
#define IPC_SMS_BASE          (41<<16)
#define IPC_CONTACTS_BASE     (42<<16)
#define IPC_SS_BASE           (43<<16)
#define IPC_SIM_BASE          (44<<16)
#define IPC_NETWORK_BASE      (45<<16)
#define IPC_DIALUP_BASE       (46<<16)
#define IPC_GCOMM_BASE		  (47<<16)

// NAAS related
#define IPC_NAAS_BASE		  (50<<16)


/************************************
 * Macros for defining status codes *
 ************************************/

typedef DWORD IPCSTATUS;

#define STATUS_BASE           (3<<24)
#define STATUS(base,name)     ((IPC_##base##_BASE)+(STATUS_BASE)+(name))

// General status definitions

#define IPC_STATUS_SUCCESS        0
#define IPC_STATUS_ERROR          1
#define IPC_STATUS_NOT_HANDLED    2 
#define IPC_STATUS_INVALIDPARAM   3

/********************************
 * Macros for defining messages *
 ********************************/

#define IPC_TYPE_MASK         (3<<24)	// General mask for getting message type from MessageID

#define IPC_REQ_BASE          (0<<24)
#define IPC_RSP_BASE          (1<<24)
#define IPC_IND_BASE          (2<<24)

#define REQ(base,message)     ((IPC_##base##_BASE)+(IPC_REQ_BASE)+(message))           
#define RSP(base,message)     ((IPC_##base##_BASE)+(IPC_RSP_BASE)+(message))           
#define IND(base,message)     ((IPC_##base##_BASE)+(IPC_IND_BASE)+(message))           


/**********************
 * General messages   *
 **********************/

#pragma pack(1)

// Definition of general IPC message head
typedef struct _IPCMESSAGEHEADER
{
	DWORD TID;
	DWORD MessageID;
	DWORD Timeout;
	DWORD Len;
} IPCMESSAGEHEADER, *PIPCMESSAGEHEADER;

typedef struct _IPCREQHEADER
{
	DWORD TID;
	DWORD MessageID;
	DWORD Timeout;
	DWORD Len;
} IPCREQHEADER, *PIPCREQHEADER;

typedef struct _IPCRSPHEADER
{
	DWORD TID;
	DWORD MessageID;
	DWORD Timeout;
	DWORD Len;
	DWORD Status;
} IPCRSPHEADER, *PIPCRSPHEADER;

typedef struct _IPCINDHEADER
{
	DWORD TID;
	DWORD MessageID;
	DWORD Timeout;
	DWORD Len;
} IPCINDHEADER, *PIPCINDHEADER;

typedef UNALIGNED struct _IPCMESSAGE
{
	IPCMESSAGEHEADER Header;
	BYTE Data[1];
} IPCMESSAGE, *PIPCMESSAGE;

typedef UNALIGNED struct _IPCREQ
{
	union {
		IPCREQHEADER REQHeader;
		IPCMESSAGEHEADER Header;
	} a;
	BYTE Data[1];
} IPCREQ, *PIPCREQ;

typedef UNALIGNED struct _IPCRSP
{
	union {
		IPCRSPHEADER RSPHeader;
		struct {
			IPCMESSAGEHEADER Header;
			DWORD Status;
		} b;
	} a;
	BYTE Data[1];
} IPCRSP, *PIPCRSP;

typedef UNALIGNED struct _IPCIND
{
	union {
		IPCINDHEADER INDHeader;
		IPCMESSAGEHEADER Header;
	} a;
	BYTE Data[1];
} IPCIND, *PIPCIND;


#pragma pack()

/**********************************************************
 * A general indication structure would be suitable for   *
 * most of the cases.                                     *
 **********************************************************/

#pragma pack(1)
typedef struct _INDGENERAL
{
	IPCINDHEADER Header;
	DWORD Parameter;
	DWORD Parameter2;
} INDGENERAL, *PINDGENERAL, IPCINDGENERAL, *PIPCINDGENERAL;
#pragma pack()

/****************************************************************
 * A few general request and response structure suitable        *
 * for many of the cases.                                       *
 * Usage example: typedef REQGENERAL REQPROFENUM, *PREQPROFENUM *
 ****************************************************************/

#pragma pack(1)
typedef struct _REQGENERAL
{
	IPCREQHEADER Header;
} REQGENERAL, *PREQGENERAL;

typedef struct _RSPGENERAL
{
	IPCRSPHEADER Header;
} RSPGENERAL, *PRSPGENERAL;


typedef struct _REQGENERALPARAM
{
	IPCREQHEADER Header;
	DWORD Parameter;
} REQGENERALPARAM, *PREQGENERALPARAM;

typedef struct _RSPGENERALPARAM
{
	IPCRSPHEADER Header;
	DWORD Parameter;
} RSPGENERALPARAM, *PRSPGENERALPARAM;

#pragma pack()

#define TYPE_OF_MESSAGE(MsgId)      (MsgId & 0x03000000)
#define ORIGIN_OF_MESSAGE(MsgId)    (MsgId & 0x00FF0000)
#define MESSAGE_OF_MESSAGE(MsgId)   (MsgId & 0x0000FFFF)

/*
 *	Messages
 */

#define REQ_IPC_NUMOFCLIENTS        REQ(IPC, 1)
#define RSP_IPC_NUMOFCLIENTS        RSP(IPC, 1)

typedef struct _RSPIPCNUMOFCLIENTS
{
	DWORD dwNumOfClients;
} RSPIPCNUMOFCLIENTS, *PRSPIPCNUMOFCLIENTS;

/*
 *	IPC STATUS Codes
 */

#define IPC_STATUS_FATAL_ERROR         ((IPCSTATUS)STATUS(IPC,1))

/*
 *	Pipe specific codes
 */

#define IPC_STATUS_PIPE_TIMEOUT        ((IPCSTATUS)STATUS(IPC,2))
#define IPC_STATUS_PIPE_EMPTY          ((IPCSTATUS)STATUS(IPC,3))
#define IPC_STATUS_PIPE_FULL           ((IPCSTATUS)STATUS(IPC,4))
#define IPC_STATUS_PIPE_LOST           ((IPCSTATUS)STATUS(IPC,5))
									   
#define IPC_STATUS_ACTION_TIMEOUT      ((IPCSTATUS)STATUS(IPC,6))
#define IPC_STATUS_REQUEST_TIMEOUT     ((IPCSTATUS)STATUS(IPC,7))

#define IPC_STATUS_NULL_MESSAGE        ((IPCSTATUS)STATUS(IPC,11))
#define IPC_STATUS_PENDING_QUEUE_FULL  ((IPCSTATUS)STATUS(IPC,12))
#define IPC_STATUS_RESPONSE_QUEUE_FULL ((IPCSTATUS)STATUS(IPC,13))

#define IPC_STATUS_INVALID_HANDLE      ((IPCSTATUS)STATUS(IPC,14))

#define IPC_STATUS_SERVER_DEAD         ((IPCSTATUS)STATUS(IPC,20))
#define IPC_STATUS_CLIENT_DEAD         ((IPCSTATUS)STATUS(IPC,21))
#define IPC_STATUS_CLIENT_EXIST        ((IPCSTATUS)STATUS(IPC,22))
#define IPC_STATUS_SERVER_EXIST        ((IPCSTATUS)STATUS(IPC,23))

#define IPC_STATUS_HUB_FULL            ((IPCSTATUS)STATUS(IPC,30))
#define IPC_STATUS_HUB_BUSY            ((IPCSTATUS)STATUS(IPC,31))

#define IPC_STATUS_BUFFER_OVERFLOW     ((IPCSTATUS)STATUS(IPC,40))
#define IPC_STATUS_NOT_IMPLEMENTED     ((IPCSTATUS)STATUS(IPC,41))
#define IPC_STATUS_NO_PROVIDER         ((IPCSTATUS)STATUS(IPC,42))

#pragma pack(1)

#define MAX_IMSI_LENGTH					8
#define MAX_RANDOM_LENGTH				16
#define MAX_SRES_LENGTH					4
#define MAX_KC_LENGTH					8

/*
 * Read IMSI.
 */

#define REQ_SIM_READIMSI					REQ(SIM,6)
#define RSP_SIM_READIMSI					RSP(SIM,6)

typedef struct
{
	IPCREQHEADER Header;
} REQSIMIMSIREAD, *PREQSIMIMSIREAD;

typedef struct
{
	IPCRSPHEADER Header;
	BYTE IMSILength;
	BYTE IMSI[MAX_IMSI_LENGTH];
} RSPSIMIMSIREAD, *PRSPSIMIMSIREAD;

/*
 * Run GSM algorithm.
 */

#define REQ_SIM_RUNGSMALG					REQ(SIM,7)
#define RSP_SIM_RUNGSMALG					RSP(SIM,7)

// Type
#define GSM_ALGORITHM							0
#define GPRS_ALGORITHM							1     // Do not use, reserved for future use

typedef struct
{
	IPCREQHEADER Header;
	BYTE Type;
	BYTE Random[MAX_RANDOM_LENGTH];
} REQSIMRUNGSMALG, *PREQSIMRUNGSMALG;

typedef struct
{
	IPCRSPHEADER Header;
	BYTE Sres[MAX_SRES_LENGTH];
	BYTE Kc[MAX_KC_LENGTH];
} RSPSIMRUNGSMALG, *PRSPSIMRUNGSMALG;

#pragma pack()

#pragma pack()

#ifdef __linux__
/* Linux stuff */
#define RSP_TIDMASK  0x0000ffff
#define MSQ_ID_MASK  0x03FF0000
#define MSQ_KEY_MASK 0x03000000
#define MSQ_KEY_HASH 0x407f98ea
#define KEY_OF_MSQ(id) (((id) & MSQ_KEY_MASK) | MSQ_KEY_HASH)
#define RSP_MSG_TYPE(id, tid)  (((id) & MSQ_ID_MASK) | ((tid) & RSP_TIDMASK))
#define REQ_MSG_TYPE(id)  ((id) & MSQ_ID_MASK)
#define IND_MSG_TYPE(id)  REQ_MSG_TYPE(id)

#pragma pack(1)
typedef struct MSQMSG {
        long int mtype;
        IPCMESSAGE ipc_msg[1];
}MSQMSG;
#pragma pack()
#endif /* #ifdef __linux */

#ifdef __cplusplus
}
#endif

#endif /* #ifndef AM_SIM_D211_H */
