/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef NAAP_DT_H
#define NAAP_DT_H

#include "naap_am.h"

#ifdef __cplusplus
extern "C" {
#endif

/* The NAAP core prints debug traces as follows:
	
   DT((u_int32_t zone, const u_int8_t* pszFormat, ...));

   for example

   DT((DT_DISCOVERY, "Received advertisement from %s", am_inet_ntoa(agent_addr))); 

   If NAAP_DEBUG is defined, DT macro will result in a call to dt().
   If NAAP_DEBUG is not defined, DT macro will not produce code.
*/

#ifdef NAAP_DEBUG
#define DT(xxx) do { dt xxx; } while (0)
#define DT_ARRAY(xxx) do { dt_array xxx; } while (0)
#define DT_ASSERT(expr,xxx) do {if(!(##expr)) dt xxx;} while(0)
#else
#define DT(xxx)
#define DT_ARRAY(xxx)
#define DT_ASSERT(expr,xxx)
#endif

/* the debug tracing function used by the NAAP Core */
void dt(u_int32_t zone, const u_int8_t* pszFormat, ...);

/* the array debug tracing function used by the NAAP Core */
void dt_array(u_int32_t zone, const u_int8_t* header_text, const u_int8_t *line_header, u_int8_t *array, u_int32_t array_len, u_int8_t output_mode);

/* DT_ARRAY output modes */
#define ARRAY_OUTPUT_ASCII	0
#define ARRAY_OUTPUT_HEX	1

/* Debug tracing "zones" */

#define DT_DISCOVERY		0x0001
#define DT_REGISTRATION		0x0002
#define DT_AUTHENTICATION	0x0004
#define DT_SOCKET			0x0008
#define DT_SIM				0x0010
#define DT_AM				0x0020 /* not used by the NAAP Core */
#define DT_MISC				0x0040
#define DT_PACKET			0x0080
#define	DT_ERROR			0x0100
#define	DT_EVENT			0x0200
#define DT_CONF				0x0400

#ifdef __cplusplus
}
#endif

#endif /* NAAP_DT_H not defined */
