/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef NAAP_EAP_MSG_H
#define NAAP_EAP_MSG_H

#include "naap_am.h"

#ifdef __cplusplus
extern "C" {
#endif
	

#if !defined(PACKED_STRUCT)
#define PACKED_STRUCT
#endif /* #if !defined(PACKED_STRUCT) */

#define EAP_CODE_REQUEST	1
#define EAP_CODE_RESPONSE	2
#define EAP_CODE_SUCCESS	3
#define EAP_CODE_FAILURE	4


#define EAP_REQ_RESP_TYPE_IDENTITY		1
#define EAP_REQ_RESP_TYPE_NOTIFICATION	2
#define EAP_REQ_RESP_TYPE_NAK			3
#define EAP_REQ_RESP_TYPE_MD5_CHALLENGE	4
#define	EAP_REQ_RESP_TYPE_OTP			5
#define EAP_REQ_RESP_TYPE_TOKEN_CARD	6
#define EAP_REQ_RESP_TYPE_GSMSIM		18

#define GSMSIM_EAP_START_SUBTYPE				1
#define GSMSIM_EAP_CHALLENGE_SUBTYPE			2
#define GSMSIM_EAP_UNKNOWN_SUBTYPE_SUBTYPE		3
#define GSMSIM_EAP_UNSUPPORTED_VERSION_SUBTYPE	4

#define NAAP_NONCEMT_LENGTH				16
#define NAAP_RAND_LENGTH				16
#define NAAP_MAC_LENGTH					16


#include "struct_pack_1.h"

struct eap_packet
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	/* data follows immediately this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_request_response
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	/* data follows immediately this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_success_failure
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_identity
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	/* identity follows immediately this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_request_response_notification
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	/* message follows immediately this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_nak
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t desired_type;
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_request_gsmsim_start
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t version_count; /* number of following version numbers */
	/* version numbers (u_int8_t) follow immediately after this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_gsmsim_start
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t version;
	u_int8_t reserved;
	u_int32_t key_lifetime_proposal;
	u_int8_t noncemt[NAAP_NONCEMT_LENGTH];
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_request_gsmsim_challenge
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t version;
	u_int8_t reserved;
	u_int32_t key_lifetime;
	u_int8_t mac_rand[NAAP_MAC_LENGTH];
	/* RANDs (u_int8_t[NAAP_RAND_LENGTH]) follow immediately this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_gsmsim_challenge
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t version;
	u_int8_t reserved;
	u_int8_t mac_sres[NAAP_MAC_LENGTH];
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_gsmsim_unknown_subtype
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t unknown_subtype;
} PACKED_STRUCT;

#include "struct_pack_default.h"


#include "struct_pack_1.h"

struct eap_response_gsmsim_unsupported_version
{
	u_int8_t code;
	u_int8_t identifier;
	u_int16_t length;
	u_int8_t type;
	u_int8_t subtype;
	u_int8_t version_count; /* number of following version numbers */
	/* version numbers (u_int8_t) follow immediately after this header */
} PACKED_STRUCT;

#include "struct_pack_default.h"

#ifdef __cplusplus
}
#endif

#endif /* #ifndef NAAP_EAP_MSG_H */
