/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef NAAP_EVENT_H
#define NAAP_EVENT_H

#include "naap_am.h"

#ifdef __cplusplus
extern "C" {
#endif


struct Event;
typedef void (*EventHandler) (struct Event*);

struct Event
{
	struct timeval when;
	EventHandler   handler;
	void*          data;
};

typedef struct Event Event;

typedef struct Event_handle
{
	Event *event;
	struct timeval timestamp;
} Event_handle;



/* print all events in the event queue to the debug log 
   This function can be used on any entity (NC, NA)
   if the implementation of the  adaptation module finds it useful */
void nc_print_events (void);

/* insert event to the event queue 
   returns a handle in ret (caller must allocate space for ret)
   returns 0 if successful 
   This function can be used on any entity (NC, NA)
   if the implementation of the  adaptation module finds it useful */
int32_t nc_add_to_events (/*IN*/ EventHandler handler, /*IN*/ int32_t milliseconds, /*IN*/ void* data,
					  /*OUT*/ Event_handle* ret);

/* dequeue event 
   This function can be used on any entity (NC, NA)
   if the implementation of the  adaptation module finds it useful */
void nc_del_from_events (/*IN*/ Event_handle* handle);

/* how many seconds until the next event in the event 
   queue should be executed? 
   Used on: NC, NA (main loop) */
struct timeval* nc_time_to_next_event (void);

/* execute all events for which the "when" field is less than 
   the current time 
   Used on: NC, NA (main loop) */
void nc_handle_events (void);

/* does such event exist 
   This function can be used on any entity (NC, NA)
   if the implementation of the  adaptation module finds it useful */
int32_t nc_is_event (/*IN*/ Event_handle* handle);

#ifdef __cplusplus
}
#endif

#endif /* NAAP_EVENT_H not defined */
