/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef NAAP_SIM_H
#define NAAP_SIM_H

#include "naap_am.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * SIM card routines used on a NAAP client
 */


/* Request user's IMSI.  the AM replies by calling nc_ans_imsi() or nc_ans_imsi_err()
   returns 0 if successful 
   Used on: NC */
int32_t am_req_imsi();

/* in response to am_req_imsi(), tell user's IMSI and NAI realm part to NAAP Core

   The IMSI is represented as specified in the GSM specification 03.03. 
   The IMSI is an ASCII string of not more than 15 characters, where
   each character represents a decimal digit (ASCII values between 0x30 and 0x39).
 
   Used on: NC */
void nc_ans_imsi(/*IN*/ u_int8_t *imsi, u_int16_t imsi_len,
				 /*IN*/ u_int8_t *realm, u_int16_t realm_len);

/* in response to am_req_imsi(), indate that imsi could not be fetched */
void nc_ans_imsi_err();


/* Request Kc and SRES that correspond to rand 
   the AM replies by calling nc_ans_kc_sres() or nc_ans_kc_sres_err()
   returns 0 if successful */
int32_t am_req_kc_sres(u_int32_t context, /*IN*/ u_int8_t* rand, u_int16_t rand_len);

/* In response to am_req_kc_sres(), give a GSM triplet to the NAAP core 
   the AM copies the context parameter from the context parameter of am_req_kc_sres */
void nc_ans_kc_sres(u_int32_t context,
					/*IN*/ u_int8_t* kc, u_int16_t kc_len,
					/*IN*/ u_int8_t* sres, u_int16_t sres_len);


/* in response to am_req_kc_sres(), indate that Kc and SRES could not be fetched 
   the AM copies the context parameter from the context parameter of am_req_kc_sres*/
void nc_ans_kc_sres_err(u_int32_t context);

#ifdef __cplusplus
}
#endif

#endif /* NAAP_SIM_H not defined */

