/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux.
 *
 * Copyright (c) 2002 Nokia. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef NAAP_UI_H
#define NAAP_UI_H

#include "naap_am.h"

#ifdef __cplusplus
extern "C" {
#endif

/* User has clicked "Connect".
   Return value: 0 if the NAAP core was in an acceptable state 
                 when the button was clicked

                 1 otherwise */
u_int8_t nc_connect_clicked();

/* User has clicked "Cancel".
   Return value: 0 if the NAAP core was in an acceptable state 
                 when the button was clicked

                 1 otherwise */
u_int8_t nc_cancel_clicked();

/* User has clicked "Disconnect".
   Return value: 0 if the NAAP core was in an acceptable state 
                 when the button was clicked

                 1 otherwise */
u_int8_t nc_disconnect_clicked();

/* the NAAP core wants the user to confirm/deny reauthentication.*/
void am_req_confirm_reauthentication();

/* the adaptation module calls nc_ans_confirm_reauthentication
   in response to am_req_confirm_reauthentication
   when the user has confirmed or denied reauthentication.
   if response != 0, user confirms reauthentication
   if response == 0, user denies reauthentication 
   Return value: 0 if the NAAP core was in an acceptable state 
                 1 otherwise */
u_int8_t nc_ans_confirm_reauthentication(int32_t response);

/* When the NAAP core isn't able to get a NAAP Reply to a deregistration
   request, the NAAP core calls am_disconnect_failed to ask the
   adaptation module whether to stay connected or forget the connection. 
   The adaptation module answers synchronously by returning STAY_CONNECTED
   or DROP_CONNECTION.
   */

typedef enum { 
    STAY_CONNECTED, 
    DROP_CONNECTION 
} Action_on_disconnect_failed;

Action_on_disconnect_failed am_disconnect_failed();

#ifdef __cplusplus
}
#endif

#endif /* NAAP_UI_H not defined */
