/*======================================================================

    Nokia 11Mbps Wireless LAN card driver for Linux

    The contents of this file are subject to the Mozilla Public
    License Version 1.1 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.

    The initial developer of the original code is David A. Hinds
    <dahinds@users.sourceforge.net>.  Portions created by David A. Hinds
    are Copyright (C) 1999 David A. Hinds.  All Rights Reserved.

    Alternatively, the contents of this file may be used under the
    terms of the GNU Public License version 2 (the "GPL"), in which
    case the provisions of the GPL are applicable instead of the
    above.  If you wish to allow the use of your version of this file
    only under the terms of the GPL and not to allow others to use
    your version of this file under the MPL, indicate your decision
    by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL.  If you do not delete
    the provisions above, a recipient may use your version of this
    file under either the MPL or the GPL.
    
======================================================================*/
/* MDU not necessary anymore in 2.6
#define D_MODULE_MAIN
*/
#include "nokia_info.h"
#include "nokia_priv.h"
#include "dserial_priv.h"
#include "dllc_priv.h"

/* one sec in jiffies */
const int DHZ = HZ;

/* externs */
/* check_wlan_gprs_concurrency is used to control WLAN/GSM/GPRS
 * simultaneous use.
 * Normally they cannot be used concurrently
 * (check_wlan_gprs_concurrency is set to 1). Using them concurrently
 * may break your pcmcia slot, so setting of check_wlan_gprs_concurrency
 * to 0 (concurrent use enabled) is done with your own risk.
 * You can control check_wlan_gprs_concurrency via module param. 
 */
extern int check_wlan_gprs_concurrency;
/* MDU replaced by the typed version */
MODULE_PARM(check_wlan_gprs_concurrency, "i");
/*static char *check_wlan_gprs_concurrency_param_name = "i";
module_param(check_wlan_gprs_concurrency_param_name, int, 0); */

/* Static functions' declarations */
static void d_free_nokia_cs (nokia_cs_t *);
static int d_init_module (void);
static void d_cleanup_module (void);
static dev_link_t * d_driver_attach (void);
static void d_driver_detach (dev_link_t *dev_link );
/* MDU move to new dev_link_t struct 
static void d_driver_release (UINT32 arg ); */
static void d_driver_release (dev_link_t *link);
static int d_driver_event (event_t event , int priority , event_callback_args_t *args );
static void d_driver_config (dev_link_t *dev_link );
/* MDU changed signature to return irqreturn_t instead of void */
static irqreturn_t d_driver_ISR (int irq , void * dev_id , struct pt_regs *regs );

static int irq_list[4] = { -1 };
static u_int irq_mask = 0xffb8;

static dev_link_t *dev_list = NULL;

static dev_info_t dev_info = "nokia_cs";

wait_queue_head_t stop_queue;
DECLARE_WAIT_QUEUE_HEAD(stop_queue);
/* MDU Not longer necessary. Provided by kernel-source */
static void cs_error_(client_handle_t handle, int func, int ret)
{
        error_info_t error_info = { func, ret };
  
        DEBUG("CS_ERROR!\n");
	pcmcia_report_error(handle, &error_info);
}

static void d_free_nokia_cs(nokia_cs_t *nokia_cs)
{
        DEBUG("In d_free_nokia_cs!\n");
  
        if(!nokia_cs) return;

        if (nokia_cs->dev && nokia_cs->dev->async_queue){
                DEBUG("Send SIGHUP to chr!\n");
                kill_fasync(&nokia_cs->dev->async_queue, SIGHUP, POLL_ERR);
        }

        if (nokia_cs->driver_priv)
                dser_tty_hangup(nokia_cs->driver_priv);

        if (nokia_cs->driver_priv->llc)
                dllc_unregister (nokia_cs->driver_priv->llc);

        /* MDU: MOD_IN_USE is not anymore user mode daemon will not be using driver
	 * safely ensured by modutils
				while(MOD_IN_USE){
                ERROR("Module in use (count: %i), waiting for user mode daemon\n", MOD_IN_USE);
                wld_wake_up_write(nokia_cs->driver_priv);
                dser_wake_up_write(nokia_cs->driver_priv);

                wait_event_interruptible(stop_queue, !MOD_IN_USE);
                DEBUG("Sleeping....\n");
                interruptible_sleep_on_timeout(&stop_queue, HZ);
        } */

        if(nokia_cs->driver_priv->mgr) {
                dmgr_delete(nokia_cs->driver_priv->mgr);
                nokia_cs->driver_priv->mgr = NULL;
        }

        if(nokia_cs->driver_priv->llc) {
                dllc_delete(nokia_cs->driver_priv->llc);
                nokia_cs->driver_priv->llc = NULL;
        }

        if(nokia_cs->dev) {
                ddev_unregister(nokia_cs->driver_priv);	
                nokia_cs->dev = NULL;
        }
  
        if(nokia_cs->driver_priv->hw) {
                dhw_delete(nokia_cs->driver_priv->hw);
                nokia_cs->driver_priv->hw = NULL;
        }

         
        //free serial port
        if(nokia_cs->driver_priv->ser) {
                dser_delete(nokia_cs->driver_priv->ser);
                nokia_cs->driver_priv->ser = NULL;
        }
    
        dfree(nokia_cs->driver_priv,sizeof(driver_priv_t));
        dfree(nokia_cs,sizeof(nokia_cs_t));
}

/*
EXPORT_NO_SYMBOLS;
*/
#ifdef MODULE_LICENSE
/* MODULE_LICENSE("Proprietary"); */
MODULE_LICENSE("GPL");
#endif

static struct pcmcia_driver nokia_driver = {
  .owner    = THIS_MODULE,
  .drv    = {
    .name = "nokia_cs",
  },
  .attach   = d_driver_attach,
  .detach   = d_driver_detach,
};

static int d_init_module(void)
{
        /* MDU not necessary anymore with new pcmcia_register_driver
	 * servinfo_t servinfo; */

        dnc_print_version();
  
        DEBUG("** Debug messages are Enabled.\n");

        if (!check_wlan_gprs_concurrency){
                NOTE("WARNING!!! WLAN/GSM/GPRS concurrent access enabled!.\n");
                NOTE("THIS MAY BREAK YOUR PCMCIA SLOT.\n");
                NOTE("USING THIS FEATURE IS DONE WITH YOUR OWN RISK.\n");
        }

        /* MDU not necessary anymore with new pcmcia_register_driver
	 * CardServices(GetCardServicesInfo, &servinfo);
        if (servinfo.Revision != CS_RELEASE_CODE) {
                ERROR("Card Services release mismatch (%x/%x).\n", servinfo.Revision, CS_RELEASE_CODE);
        }

        return register_pcmcia_driver(&dev_info, &d_driver_attach, &d_driver_detach); */
	return pcmcia_register_driver(&nokia_driver);
}

static void d_cleanup_module(void)
{
        NOTE("Unloading Nokia D211 module.\n");
  
        pcmcia_unregister_driver(&nokia_driver);
  
        DEBUG("Looping thru list..\n");
        
        while(dev_list) {
                DEBUG("Loop..\n");
                if (dev_list->state & DEV_CONFIG)
                        d_driver_release(dev_list);
                d_driver_detach(dev_list);
        }

        DEBUG("cleanuploop Done.\n");
  
        if(dev_list) {
                ERROR("Nokia D211 instances remain when unloading module.\n");
        }

        return;
}

static dev_link_t *d_driver_attach(void)
{
        nokia_cs_t *nokia_cs;
        dev_link_t *dev_link = NULL;
        client_reg_t client_reg;
        int i, result, ret;
        dev_node_t *dev_node = NULL;
        char mac_addr[MAC_ADDR_SIZE] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
      
        NOTE("Card Inserted. Initializing...\n");
  
        /* Do the basic structure */
        nokia_cs = kmalloc(sizeof(nokia_cs_t),GFP_KERNEL);
        if(!nokia_cs) {
                ERROR("Unable to alloc memory for main structure.\n");
                return NULL;
        }
  
        memset(nokia_cs,0,sizeof(nokia_cs_t));

        nokia_cs->driver_priv = kmalloc(sizeof(driver_priv_t),GFP_KERNEL);
        if(!nokia_cs->driver_priv) {
                ERROR("Unable to alloc memory for driver structure.\n");
                return NULL;
        }
  
        memset(nokia_cs->driver_priv,0,sizeof(driver_priv_t));

        nokia_cs->driver_priv->nokia_cs = (void*) nokia_cs;

        init_waitqueue_head(&nokia_cs->wld_wait);
     
        dev_link = &nokia_cs->link;
        dev_link->priv = nokia_cs;
        
        dev_node= &nokia_cs->node;
  
        /* Initialize the dev_link structure */
				/* MDU: dev-link does not contain release att anymore */ /*
        dev_link->release.function = &d_driver_release;
        dev_link->release.data = (UINT32) dev_link; */
          
        /* Interrupt setup */
        dev_link->irq.Attributes = IRQ_TYPE_EXCLUSIVE | IRQ_HANDLE_PRESENT;
        dev_link->irq.IRQInfo1 = IRQ_INFO2_VALID | IRQ_LEVEL_ID;
        if (irq_list[0] == -1) {
                dev_link->irq.IRQInfo2 = irq_mask;
        } else {
                for (i = 0; i < 4; i++) {
                        dev_link->irq.IRQInfo2 |= 1 << irq_list[i];
                }
        }
        dev_link->irq.Instance = dev_link;
        dev_link->irq.Handler = &d_driver_ISR;
  
        dev_link->conf.Attributes = CONF_ENABLE_IRQ;
        dev_link->conf.IntType = INT_MEMORY_AND_IO;
        dev_link->conf.Vcc = 50;
        dev_link->conf.Vpp1 = 0;
        dev_link->conf.Vpp2 = 0;
  
        dev_link->io.Attributes2 = 0;
        dev_link->io.BasePort2 = 0;
        dev_link->io.IOAddrLines = 4;
        dev_link->io.NumPorts2 = 0;
  
        /* Register with the Card Services */
        dev_link->next = dev_list;
        dev_list = dev_link;
        client_reg.Attributes = INFO_IO_CLIENT | INFO_CARD_SHARE;
        client_reg.dev_info = &dev_info;
  
        /* Supported events */
        client_reg.EventMask =
                CS_EVENT_CARD_INSERTION | CS_EVENT_CARD_REMOVAL |
                CS_EVENT_RESET_PHYSICAL | CS_EVENT_CARD_RESET |
                CS_EVENT_PM_SUSPEND | CS_EVENT_PM_RESUME;

        client_reg.event_handler = &d_driver_event;
        client_reg.event_callback_args.client_data = dev_link;
        client_reg.Version = 0x0210;

        /* Create the llc instance */
        nokia_cs->driver_priv->llc = dllc_new(nokia_cs->driver_priv);
        if(!nokia_cs->driver_priv->llc) {
                ERROR("No llc instance.\n");
                goto error;
        }

        if (dllc_register (nokia_cs->driver_priv->llc)){
                ERROR("Unable to register dllc.\n"); 
                goto error;
        }

        /* Copy the device name for PCMCIA */
        strcpy(dev_node->dev_name, nokia_cs->driver_priv->llc->dev->name);
  
        /* Now register to PCMCIA */
	/* MDU replaced CardServices ofr the relevant pcmcia_function */
        ret = pcmcia_register_client(&dev_link->handle, &client_reg);
        if (ret != CS_SUCCESS) {
                cs_error_(dev_link->handle, RegisterClient, ret);
                d_driver_detach(dev_link);
                return NULL;
        }
  
        /* Create the hw instance. It takes care of card initialization too */
        nokia_cs->driver_priv->hw = dhw_new(nokia_cs->driver_priv,dev_link->io.BasePort1,dev_link->irq.AssignedIRQ);
        if(!nokia_cs->driver_priv->hw) {
                ERROR("No hw instance.\n");
                goto error;
        }
  
        /* Since HW is initialized, we can get the MAC address */
        dnc_get_my_mac(nokia_cs->driver_priv, mac_addr);
        dllc_set_my_mac_addr(nokia_cs->driver_priv, mac_addr);

        /* Create the mgr instance */
        nokia_cs->driver_priv->mgr = dmgr_new(nokia_cs->driver_priv);
        if(!nokia_cs->driver_priv->mgr) {
                ERROR("No mgr instance.\n");
                goto error;
        }

        /* Create character device */
        result = ddev_register(nokia_cs->driver_priv);
        if(!result) {
                ERROR("Unable to start character device.\n"); 
                goto error;
        }

        /* Create the serial instance */
        nokia_cs->driver_priv->ser = dser_new(nokia_cs->driver_priv);
        if(!nokia_cs->driver_priv->ser) {
                ERROR("No serial instance.\n");
                goto error;
        }

        /* And we're done */
        DEBUG("Module initialization complete.\n");
  
        return dev_link;
  
  error:
  
        d_driver_detach(dev_link);
        return NULL;
}

void d_driver_detach(dev_link_t *dev_link)
{
        dev_link_t **dev_linkp;
  
        DEBUG("d_driver_detach. Freeing resources.\n");
  
        if(!dev_link) return;
  
        DEBUG("Locating device structure.\n");
  
        /* Locate device structure */
        for (dev_linkp = &dev_list; *dev_linkp; dev_linkp = &(*dev_linkp)->next) {
                if (*dev_linkp == dev_link) break;
        }
        if (*dev_linkp == NULL) {
                DEBUG("PCMCIA detach failed. '%s' is not in the list.\n",dev_link->dev->dev_name);
                return;
        }
  
        DEBUG("Struct located.\n");
  
        if (dev_link->state & DEV_CONFIG) {
                d_driver_release(dev_link);
                if(dev_link->state & DEV_STALE_CONFIG) {
                        DEBUG("PCMCIA detach postponed, '%s' still locked.\n", dev_link->dev->dev_name);
                        dev_link->state |= DEV_STALE_LINK;
                        return;
                }
        }
  
        /* Break the link with Card Services */
        if (dev_link->handle){
		/* MDU replaced CardServices ofr the relevant pcmcia_function */
                pcmcia_deregister_client(dev_link->handle);
                DEBUG ("Client deregistered\n");
        }
        dev_link->handle = 0;
  
        DEBUG("Card services freed.\n");
  
        /* Unlink device structure, and free it */
        *dev_linkp = dev_link->next;
  
        /* Free Nokia specific things */
        DEBUG("Freeing Nokia_D211...\n");
  
        if(dev_link->priv) {
                d_free_nokia_cs(dev_link->priv);
        }
        dev_link->priv = NULL;
  
        DEBUG("Detach complete!\n");
}

/* MDU change signature
void d_driver_release(UINT32 arg) */
void d_driver_release(dev_link_t *dev_link)
{
        /* MDU
	dev_link_t *dev_link = (dev_link_t *)arg; */
  
        DEBUG("d_driver_release called.\n");
  
        if(!dev_link) return;
  
        if (dev_link->open) {
                DEBUG("Nokia D211 instance release postponed, '%s' still open\n",
                      dev_link->dev->dev_name);
                dev_link->state |= DEV_STALE_CONFIG;
                return;
        }
  
        dev_link->dev = NULL;
  
        if (dev_link->win) {
		/* MDU replaced CardServices ofr the relevant pcmcia_function */
                pcmcia_release_window(dev_link->win);
        }
	/* MDU replaced CardServices ofr the relevant pcmcia_function */
        pcmcia_release_configuration(dev_link->handle);
        if (dev_link->io.NumPorts1) {
		/* MDU replaced CardServices ofr the relevant pcmcia_function */
                pcmcia_release_io(dev_link->handle, &dev_link->io);
        }
        if (dev_link->irq.AssignedIRQ) {
		/* MDU replaced CardServices ofr the relevant pcmcia_function */
                pcmcia_release_irq(dev_link->handle, &dev_link->irq);
        }  
        dev_link->state &= ~DEV_CONFIG;
        if (dev_link->state & DEV_STALE_LINK) {
                d_driver_detach(dev_link);
        }
} 


static int d_driver_event(event_t event, int priority, event_callback_args_t *args)
{
        dev_link_t *dev_link;
        nokia_cs_t *nokia_cs;
        int action = 0;

        if (!args) return 0;
        dev_link = args->client_data;
        if (!dev_link) return 0;
        nokia_cs = dev_link->priv;
        if(!nokia_cs) return 0;
  
        switch (event) {
            case CS_EVENT_CARD_REMOVAL:
                    dev_link->state &= ~DEV_PRESENT;
                    if (dev_link->state & DEV_CONFIG) {
                            nokia_cs->driver_priv->stop = 1;
                          /*  dev_link->release.expires = jiffies + HZ/20;
                            add_timer(&dev_link->release); MDU */
			    d_driver_release(dev_link);
                    }
                    action = CARD_REMOVAL;
                    break;
            case CS_EVENT_CARD_INSERTION:
                    dev_link->state |= DEV_PRESENT | DEV_CONFIG_PENDING;
                    /* nokia_cs->bus = args->bus; MDU: bus operations removed */
                    d_driver_config(dev_link);
                    action = CARD_INSERTION;
                    break;
            case CS_EVENT_PM_SUSPEND:
                    dev_link->state |= DEV_SUSPEND;
                    action = SUSPEND;
            case CS_EVENT_RESET_PHYSICAL:
                    nokia_cs->driver_priv->stop = 1;
                    if (dev_link->state & DEV_CONFIG) {
			/* MDU replaced CardServices ofr the relevant pcmcia_function */
                        pcmcia_release_configuration(dev_link->handle);
                    }
                    if (action == 0) 
                            action = RESET_PHYSICAL;
                    break;
            case CS_EVENT_PM_RESUME:
                    dev_link->state &= ~DEV_SUSPEND;
                    action = RESUME;
            case CS_EVENT_CARD_RESET:
                    if (dev_link->state & DEV_CONFIG) {
			/* MDU replaced CardServices ofr the relevant pcmcia_function */
                        pcmcia_request_configuration(dev_link->handle, &dev_link->conf);
                    }
                    nokia_cs->driver_priv->stop = 0;
                    if (action == 0) 
                            action = CARD_RESET;
                    break;
        }
        dnc_pcmcia_action(nokia_cs->driver_priv, action);

        return 0;
}


/* MDU replaced CardServices ofr the relevant pcmcia_function 
#define CS_CHECK(fn, args...) \
while ((last_ret=CardServices(last_fn=(fn),args))!=0) goto error

#define CFG_CHECK(fn, args...) \
if (CardServices(fn, args) != 0) goto next */

#define CS_CHECK(fn, ret) \
do { last_fn = (fn); if ((last_ret = (ret)) != 0) goto error; } while (0)

static void d_driver_config(dev_link_t *dev_link)
{
        client_handle_t client_handle;
        nokia_cs_t *nokia_cs;
        tuple_t tuple;
        cisparse_t cisparse;
        int last_fn, last_ret;
        unsigned char buf[254];
        unsigned char conf_buf[254];
        config_info_t config_info;
        win_req_t win_req;

        if (!dev_link) return;
        client_handle = dev_link->handle;
        nokia_cs = dev_link->priv;
        if (!nokia_cs) return;
        if (!nokia_cs->driver_priv) return;
  
        
        DEBUG("d_driver_config(0x%p)\n",dev_link);

        tuple.Attributes    = 0;
        tuple.DesiredTuple  = CISTPL_CONFIG;
        tuple.Flags         = 0;
        tuple.TupleData     = buf;
        tuple.TupleDataMax  = sizeof(buf);
        tuple.TupleOffset   = 0;

        CS_CHECK(GetFirstTuple, pcmcia_get_first_tuple(client_handle, &tuple));
        CS_CHECK(GetTupleData, pcmcia_get_tuple_data(client_handle, &tuple));
        CS_CHECK(ParseTuple, pcmcia_parse_tuple(client_handle, &tuple, &cisparse));
  
        dev_link->conf.ConfigBase = cisparse.config.base;
        dev_link->conf.Present = cisparse.config.rmask[0];

        dev_link->state |= DEV_CONFIG;

        /* Lookup manfid */
        tuple.DesiredTuple = CISTPL_MANFID;
        CS_CHECK(GetFirstTuple, pcmcia_get_first_tuple(client_handle, &tuple));
        CS_CHECK(GetTupleData, pcmcia_get_tuple_data(client_handle, &tuple));
        CS_CHECK(ParseTuple, pcmcia_parse_tuple(client_handle, &tuple, &cisparse));
        if (cisparse.manfid.manf != MANFID_NOKIA){
                ERROR("Unknown manufacturer 0x%hX\n", cisparse.manfid.manf);
                goto error;
        }
        nokia_cs->driver_priv->card_type = cisparse.manfid.card;
  
        /* Look up the Vcc */
        CS_CHECK(GetConfigurationInfo, pcmcia_get_configuration_info(client_handle, &config_info));
        dev_link->conf.Vcc = config_info.Vcc;
  
        tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
        CS_CHECK(GetFirstTuple, pcmcia_get_first_tuple(client_handle, &tuple));
  
        while (TRUE) {
    
                cistpl_cftable_entry_t cftable_entry= { 0 };
                cistpl_cftable_entry_t *cft = &(cisparse.cftable_entry);
    
                if (pcmcia_get_tuple_data(client_handle, &tuple) != 0 ||
			pcmcia_parse_tuple(client_handle, &tuple, &cisparse) != 0)
			goto next;
   		 
                if (cft->flags & CISTPL_CFTABLE_DEFAULT) 
                        cftable_entry = *cft;
                if (cft->index == 0)
                        goto next;

                /* Continue */
                dev_link->conf.ConfigIndex = cft->index;

                dev_link->conf.Vpp1 = 0;
                dev_link->conf.Vpp2 = 0;
                dev_link->io.NumPorts1 = 0;
                dev_link->io.NumPorts2 = 0;
    
                /* interrupt */
                if (cftable_entry.irq.IRQInfo1) {
                        dev_link->conf.Attributes |= CONF_ENABLE_IRQ;
                }
    
                /* IO window settings */
                if (cftable_entry.io.nwin > 0) {
                        dev_link->io.Attributes1 = IO_DATA_PATH_WIDTH_AUTO;
                        if (!(cftable_entry.io.flags & CISTPL_IO_8BIT)) {
                                dev_link->io.Attributes1 = IO_DATA_PATH_WIDTH_16;
                        }
                        if (!(cftable_entry.io.flags & CISTPL_IO_16BIT)) {
                                dev_link->io.Attributes1 = IO_DATA_PATH_WIDTH_8;
                        }
                        dev_link->io.IOAddrLines = cftable_entry.io.flags & CISTPL_IO_LINES_MASK;
                        dev_link->io.BasePort1 = cftable_entry.io.win[0].base;
      
                        dev_link->io.NumPorts1 = cftable_entry.io.win[0].len;
                        if(dev_link->io.NumPorts1==0) {
                                dev_link->io.NumPorts1 = 16;
                        }
                        if (cftable_entry.io.nwin > 1) {
                                dev_link->io.Attributes2 = dev_link->io.Attributes1;
                                dev_link->io.BasePort2 = cftable_entry.io.win[1].base;
                                dev_link->io.NumPorts2 = cftable_entry.io.win[1].len;
                        }
                }
    
                if (pcmcia_request_io(dev_link->handle, &dev_link->io) != 0)
			goto next;

                break;
          next:
                CS_CHECK(GetNextTuple, pcmcia_get_next_tuple(client_handle, &tuple));
        }
  
        if (dev_link->conf.Attributes & CONF_ENABLE_IRQ)
                CS_CHECK(RequestIRQ, pcmcia_request_irq(dev_link->handle, &dev_link->irq));
  
        CS_CHECK(RequestConfiguration, pcmcia_request_configuration(dev_link->handle, &dev_link->conf));
  
        nokia_cs->node.major = nokia_cs->node.minor = 0;
        dev_link->dev = &nokia_cs->node;
  
        sprintf(&conf_buf[0],"Config: index 0x%02x: Vcc %d.%d", dev_link->conf.ConfigIndex, dev_link->conf.Vcc/10, dev_link->conf.Vcc%10);
  
        if (dev_link->conf.Attributes & CONF_ENABLE_IRQ) {
                sprintf(&conf_buf[strlen(conf_buf)],", irq %d", dev_link->irq.AssignedIRQ);
        }
        if (dev_link->io.NumPorts1) {
                sprintf(&conf_buf[strlen(conf_buf)],", io 0x%04x-0x%04x", dev_link->io.BasePort1, dev_link->io.BasePort1+dev_link->io.NumPorts1-1);
        }
        if (dev_link->win) {
                sprintf(&conf_buf[strlen(conf_buf)],", mem 0x%06lx-0x%06lx", win_req.Base, win_req.Base+win_req.Size-1);
        }

        NOTE ("%s\n",conf_buf);

        dev_link->state &= ~DEV_CONFIG_PENDING;
  
        return;
  
  error:
  
        cs_error_(dev_link->handle, last_fn, last_ret);
        d_driver_release(dev_link);
  
        return;
}

/* chenged return of void to irqreturn_t */
static irqreturn_t d_driver_ISR (int irq, void* dev_id, struct pt_regs *regs)
{
        nokia_cs_t *nokia_cs;
        if(!dev_id){
                ERROR ("dev_id NULL\n");
                goto exit;
        }
        
        nokia_cs = ((dev_link_t *) dev_id)->priv;
        if(!nokia_cs){
                ERROR ("nokia_cs NULL\n");
                goto exit;
        }
        if(!nokia_cs->driver_priv){
                ERROR ("driver_priv NULL\n");
                goto exit;
        }
  
        // Make some necessary checks
        if(!(((dev_link_t *) dev_id)->state & DEV_PRESENT)){
                DEBUG ("dev not present\n");
                goto exit;
        }
        dnc_ISR(nokia_cs->driver_priv);
        return IRQ_HANDLED;

  exit:
        DEBUG("Interrupt handling problem!!\n");
        return IRQ_NONE;
}

module_init(d_init_module);
module_exit(d_cleanup_module);










