/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 * 
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#ifndef D_TOOLS_H
#define D_TOOLS_H

#pragma pack(1)

#ifndef NULL
#define NULL ((void *)0)
#endif

#define DEVICE_NR(minor)        ((minor)>>4)
#define MAXDEVC 8

#define DDEV_MAXITEMS 100

#define MSG_NORMAL         0
#define MSG_HIGH_PRIORITY  1

typedef struct ddev_item
{
        INT16 minor; // for what minor device to...
        INT16 type;  //type of the message
        UINT32 len;
        UINT8 *data;

        struct ddev_item *next;
} ddev_item_t;

// And our basic configuration
typedef struct ddev
{
        char name[32];
        INT16 major;
        INT16 minor;

        //read queue
        ddev_item_t *itemroot;

        struct fasync_struct *async_queue;
} ddev_t;

#pragma pack()

#ifdef D_DEBUG
#define ERROR(args...) dprintk("<3>" "D211_Error: " args)
#define NOTE(args...)  dprintk("<5>" "D211: " args)
#else
#define ERROR(args...) dprintk("<3>" "D211_Error: " args)
#define NOTE(args...)  dprintk("<5>" "D211: " args)
#endif

#ifdef D_DEBUG
#define DEBUG(args...) dprintk("<5>" "D211_Debug: " args)
#define DEBUG_ADD(args...) dprintk(args)
#else
#define DEBUG(args...)
#define DEBUG_ADD(args...)
#endif

// Memitem used by buffer space allocs
typedef struct dt_memitem {
        UINT8 *data;
        UINT32 size;

        struct dt_memitem *next;
} dt_memitem_t;

typedef struct dt_buf {
        UINT8 *data;
        INT16 length;
        INT16 maxsize;

} dt_buf_t;

INT16 dt_get16 (void *q );
void * dt_getspace (UINT32 size );
void dt_freespace (void *p );
dt_buf_t * dt_newbuf (UINT32 maxsize );
void dt_freebuf (dt_buf_t *buf );
UINT8 * dt_bufdata (dt_buf_t *buf );
INT16 dt_buflen (dt_buf_t *buf );
void dt_addbuf8 (dt_buf_t *buf , UINT8 x );
void dt_addbuf16 (dt_buf_t *buf , UINT16 x );
void dt_addbufn (dt_buf_t *buf , UINT8 *d , INT16 len );
char * dt_parse_tag (char *s , char *tag, char *tmp );
UINT8 dt_hexstrtochar (char *str );
int dt_readhex (char *out , char *data , int len );
INT32 dt_strtolong (char *str );
UINT8 * dt_printmac (UINT8 *src, UINT8 *mac_buf);
UINT8 * dt_printip (UINT32 src, UINT8 *ip_buf);
void dt_printhex (UINT8 *src , INT16 len );
INT16 dt_strcasecmp (UINT8 *a , UINT8 *b );
UINT8 * dt_load_file (char *filename, int *length);
void wld_signal_to_user(UINT32 priv);
void wld_wake_up_write(driver_priv_t* priv);
int ddev_register (driver_priv_t*);
int ddev_unregister (driver_priv_t*);
void* dmalloc_atomic(INT32 size);
void* dmalloc_kernel(INT32 size);
void* dmemcpy(void *dest, const void *src, INT32 size);
void* dmemmove(void *dest, const void *src, INT32 size);
void dfree(const void *o, INT32 s);
int dmemcmp(const void *s1, const void *s2, INT32 size);
void* dmemset(void *s1,int s2, INT32 size);
unsigned long d_copy_to_user(void* to, const void* from, unsigned long count);
unsigned long d_copy_from_user(void* to, const void* from, unsigned long count);
char* dstrcpy(char *s1,const char *s2);
char* dstrncpy(char *s1,const char *s2, INT32 size);
int dstrncmp(const char *s1,const char *s2, INT32 size);
INT32 dstrlen(const char *s1);
INT32 dstrnlen(const char *s1, int size);
int dsprintf(char * buf, const char * fmt, ...);
char* dstrcat(char *s1, const char *s2);
void d_udelay(unsigned long s1);
void d_mdelay(unsigned long s1);
/*Timer support*/
void* dt_init_timer (void (*function)(UINT32), void *data);
int dt_mod_timer(void* timer_list, unsigned long expires);
int dt_del_timer(void* timer_list);
int dt_free_timer(void* timer_list);
/*spinlock support*/
void* dt_init_lock(void);
void dt_lock (void* lock);
int dt_is_locked (void* lock);
void dt_unlock (void* lock);
void dt_lock_bh (void* lock);
void dt_unlock_bh (void* lock);
void dt_lock_irqsave (void* lock, unsigned long *flags);
void dt_unlock_irqrestore (void* lock, unsigned long *flags);
void dt_del_lock (void* lock);
/*tasklet support*/
void* dt_init_tasklet (void (*function)(UINT32), unsigned long data);
void dt_schedule_tasklet(void* tasklet);
void dt_enable_tasklet(void* tasklet);
void dt_disable_tasklet(void* tasklet);
void dt_kill_tasklet(void* tasklet);
/*waitqueue support*/
void* dt_init_waitqueue(void);
INT32 dt_interruptible_sleep_on_timeout(void* queue, long timeout);
void dt_wake_up_interruptible(void* queue);
void dt_delete_waitqueue(void* queue);
int dt_interruptible_sleep_on_state_timeout(void* queue, int* statevar, long timeout);

INT dt_in_interrupt (void);
INT dt_in_irq (void);

int dprintk(const char *fmt, ...);
unsigned short dinw(UINT32 base);
void doutw(UINT32 base,UINT32 value);

/* Handoff triggers support */
void dt_layer2_handoff(driver_priv_t* priv, int type);
#define LINKLAYER_HANDOFF_TYPE    0x0000ffff
#define LINKLAYER_HANDOFF_BEGIN   0x00000001
#define LINKLAYER_HANDOFF_DONE    0x00000002
#define LINKLAYER_HANDOFF_SUBTYPE 0xffff0000
#define WLAN_HANDOFF              0x00010000
#define GSM_HANDOFF               0x00020000

#endif
















