/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 * 
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

/***************************************************************************
 *
 * Define the nokia_cs data structure and basic 
 * configuration 
 *
 **************************************************************************/

#ifndef D_NOKIA_INFO_H
#define D_NOKIA_INFO_H

/* System specific includes */
// Do not set Linux version

#ifndef D_MODULE_MAIN
#define __NO_VERSION__
#endif

/* Module related headers */
#include <linux/module.h>
#include <linux/version.h>
#include <linux/moduleparam.h>

/* Standard driver includes */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/major.h>
#include <linux/tty.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <linux/errno.h>
#include <linux/poll.h>

/* Ethernet and network includes */
#include <linux/netdevice.h>
#include <linux/if_ether.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/if_arp.h>
#include <linux/ioport.h>
#include <linux/timer.h>

/* Card and Driver services includes */
#include <pcmcia/version.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/cs.h>
#include <pcmcia/cisreg.h>
#include <pcmcia/ds.h>
#include <pcmcia/ciscode.h>

/* serial port includes */
#include <linux/serial.h>

/* our includes */
#include "nokia_priv.h"
#include "dllc.h"
#include "dserial.h"
#include "dtools.h"
#include "dmac.h"

#pragma pack(1)

/* Global defined settings */

/*
  This is our basic nokia_cs structure. 

  These are all saved per card, so if you use two
  cards at the same time, they do not overlap
 */

typedef struct nokia_cs_info
{
        dev_link_t link;
        dev_node_t node;
	/* bus opreations removed
	struct bus_operations *bus; */

        struct ddev *dev;
        // Event wait queue
        wait_queue_head_t wld_wait;

        /* nokia specific */        
        struct driver_priv *driver_priv;       
} nokia_cs_t;

#pragma pack()

#endif









