/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 * 
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

/***************************************************************************
 *
 * Define the nokia_cs data structure and basic 
 * configuration 
 *
 **************************************************************************/

#ifndef D_NOKIA_PRIV_H
#define D_NOKIA_PRIV_H

/* our includes */
#include "types.h"

/* Global defined settings */
#define D_FIRMWAREFILE_D211 "/etc/pcmcia/bin/smac2.bin"
#define MANFID_NOKIA           0x0124

extern unsigned long volatile jiffies;
extern const int DHZ;  /* one sec in jiffies */

/* PCMCIA actions */
#define CARD_REMOVAL   1
#define CARD_INSERTION 2
#define SUSPEND        3
#define RESET_PHYSICAL 4
#define RESUME         5
#define CARD_RESET     6

/*
  This is our driver info structure holding
  all the mac/hw etc. instances. 

  These are all saved per card, so if you use two
  cards at the same time, they do not overlap
 */

struct dhw;
struct dllc;
struct dmgr;
struct ddev;
struct dser;

typedef struct driver_priv
{
        void *nokia_cs; // back pointer
        unsigned short card_type;
        int stop;
        struct dhw  *hw;
        struct dllc *llc;
        struct dser *ser;
        struct dmgr *mgr;
} driver_priv_t;

/* function declarations */
/* dhw initialization and cleanup functions */
struct dhw *dhw_new (driver_priv_t*, UINT32 base, UINT32 irq);
void dhw_delete (struct dhw*);
/* dmgr initialization and cleanup functions */
struct dmgr *dmgr_new (driver_priv_t*);
void dmgr_delete (struct dmgr*);
/* set multicast addresses to the card */
void dnc_set_multicast_addresses(driver_priv_t*, int mc_count, UINT8* mac_addr_list, int length);
/* transmit a frame to WLAN */
INT16 dnc_tx_frame (driver_priv_t*, UINT8 *dest , UINT8 *src , INT16 type , UINT8 *data , UINT32 ref, INT32 length);
/* transmit a serial frame to GSM/GPRS */
int dnc_serial_send(driver_priv_t *, const char *buf, int count);
/* send a signal to modem */
void dnc_serial_signal(driver_priv_t* priv, int signal);
/* private character device functions */
int dnc_ioctl (driver_priv_t*, unsigned int cmd, unsigned long arg);
int dnc_read(driver_priv_t*, char *buf, int count);
int dnc_write(driver_priv_t*, const char *buf, int count);
/* procfile support */
INT16 dnc_proc_status (driver_priv_t*, UINT8 *buf , INT16 bufsize );
/* prints binary part version */
void dnc_print_version(void);
/* get current mac address */
void dnc_get_my_mac (driver_priv_t*, char* mac);
/* interrupt handler */
void dnc_ISR (driver_priv_t*);
/* PCMCIA actions */
void dnc_pcmcia_action(driver_priv_t* priv, int action);

#include "dtools.h"
#include "dmac.h"

#endif














