/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
}
#include <ipcbase.h>
//WLAN
#include <pwepreq.h>
#include <wlanctlreq.h>
#include <wlanmibreq.h>
#include <mibreq.h>
#include <wlanreq.h>
//GSM
#include <sim.h>
#include <sms.h>
#include <net.h>
#include <phone.h>
#include <dialup.h>
#include <contacts.h>
//local
#include "sndmsg.h"
#include "nokia_ctl.h"

int parse_main_args (int command, union args arg){
        int SEQ_TID;
        static int seed = 1;
        char buffer[BUF_SIZE];
        PIPCMESSAGE pIpc = (PIPCMESSAGE) buffer;
        PIPCRSP pIpc_rsp = (PIPCRSP) buffer;
        int j, len;
        char x[3];
        
        bzero(buffer,sizeof(buffer));

        // Use /dev/random only for the first seed
        if (seed == 1) {
                if (!read_random(x,3)) {
                        return -1;
                }
                seed = *(int*)x;
        }

        srand(++seed);
        
        //SEQ_TID cannot be zero, this is very very rare, but...
        do {
                SEQ_TID = rand();
        } while (SEQ_TID == 0); 
        
        DBG("cmd: %d\n", command);

        switch (command){
            case ENABLE_WLAN: //enable_wlan
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANCTL_ENABLE;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET);
            }
            break;
            case DISABLE_WLAN: //disable_wlan
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANCTL_DISABLE;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET);
            }
            break;
            case ENABLE_GSM: //enable_gsm
            {
                    PREQSIMPINVERIFY pIpcSV=(PREQSIMPINVERIFY)pIpc;
                    pIpcSV->Header.TID = SEQ_TID;
                    pIpcSV->Header.MessageID = REQ_SIM_PINVERIFY;
                    pIpcSV->Header.Timeout = 0;
                    pIpcSV->Header.Len = sizeof(REQSIMPINVERIFY);
                    pIpcSV->Type = PIN;
                    strncpy((char*)pIpcSV->Pin, arg.pincode, strlen(arg.pincode));
            }
            break;
            case NETNAMES: //netnames
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;                    
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(SSID) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SET_NETNAMES;

                    pIpcMS->Data[0] = strlen(arg.netname);
                    strncpy((char *)&pIpcMS->Data[1], arg.netname,strlen(arg.netname));
            }
            break;        
            case CASE: //case
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET);
                    pIpcMS->Id = MIB_WLAN_SET_CASESENSITIVE;
                    pIpcMS->Data[0] = arg.casesensitive;
            }
            break;        
            case OPMODE: //opmode
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET);
                    pIpcMS->Id = MIB_WLAN_SET_OPMODE;
                    pIpcMS->Data[0] = arg.opmode;
            }
            break;
            case SECPOLICY: //secpolicy
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(unsigned long) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SEC_POLICY;
                    *((unsigned int*) pIpcMS->Data) = arg.weppolicy;
            }
            break;
            case SECMODE: //secmode
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(unsigned long) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SEC_MODE;
                    *((unsigned int*) pIpcMS->Data ) = arg.wepmode;
            }
            break;
            case SECSHAREDKEYS: //secsharedkeys
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    PWEPDEFKEYSSID pwdks;
                    PWEPDEFAULTKEYS pwdk;
                    int dl;

                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SEC_SHAREDKEYS;

                    if (strlen(arg.wep_args.netname) == 0)
                            //Unset keys 
                            break;

                    /* key set count (nmbr of 4-key sets), 
                     * currently only one supported */ 
                    *((unsigned int*) pIpcMS->Data ) = 1; 

                    pwdks = (PWEPDEFKEYSSID) &pIpcMS->Data[4];

                    pwdks->Len = strlen(arg.wep_args.netname); //SSID length
                    memcpy (pwdks->Data, arg.wep_args.netname, pwdks->Len);

                    pwdk = (PWEPDEFAULTKEYS)((char*)pwdks + sizeof(WEPDEFKEYSSID) + pwdks->Len - 1);
                    pwdk->Active = arg.wep_args.active;
                    
                 
                    for (j = 0, dl = 0; j < 4; j++)
                    {
                            if (arg.wep_args.keylen[j] != 0 ) {
                                    pwdk->KeyLen[j] = arg.wep_args.keylen[j];
                                    memcpy (pwdk->KeyData + dl, &arg.wep_args.keydata[j*16], arg.wep_args.keylen[j]);
                                    dl += pwdk->KeyLen[j];
                            }
                    }
                    for (j=0; j<64; j++) {
                            DBG("%02x ",pwdk->KeyData[j]);
                            if (j==15 || j==31 || j==47 || j==63) DBG("\n");
                    }
                    for (j=0;j<4;j++) {
                            DBG("%02x ",pwdk->KeyLen[j]);
                    }
                    DBG("\n");
                    pIpcMS->Header.Len +=  sizeof(unsigned long) + 1 + strlen(arg.wep_args.netname) + 1 + 4 + dl;
            }
            break;
            case MINCHANNEL: //minchannel
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(unsigned long) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SET_MINCHANNEL;
                    *((unsigned int*) pIpcMS->Data ) = arg.minchannel;
            }
            break;
            case MAXCHANNEL: //maxchannel
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(unsigned long) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SET_MAXCHANNEL;
                    *((unsigned int*) pIpcMS->Data ) = arg.maxchannel;
            }
            break;
            case RESTRICTCHANNEL: //restrictchannel
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;        
                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET) + sizeof(unsigned long) - sizeof(char);
                    pIpcMS->Id = MIB_WLAN_SET_RESTRICTCHANNEL;
                    *((unsigned int*) pIpcMS->Data ) = arg.restrictchannel;
            }
            break;
            case SECSTATIONKEY: //secpersonalkey
            {
                    PREQMIBSET pIpcMS = (PREQMIBSET)pIpc;

                    pIpcMS->Header.TID = SEQ_TID;
                    pIpcMS->Header.MessageID = REQ_WLANMIB_SET;
                    pIpcMS->Header.Timeout = 0;
                    pIpcMS->Header.Len = sizeof(REQMIBSET);
                    pIpcMS->Id = MIB_WLAN_SEC_STATIONKEY;
                    
                    memcpy (pIpcMS->Data, &arg.wep_args.keydata[0], arg.wep_args.keylen[0]);
                    pIpcMS->Header.Len +=  arg.wep_args.keylen[0] - sizeof(char);
            }
            break;
            case CHANGEPIN: //changePIN
            {
                    PREQSIMPINCHANGE pIpcSC = (PREQSIMPINCHANGE)pIpc;
                    pIpcSC->Header.TID = SEQ_TID;
                    pIpcSC->Header.MessageID = REQ_SIM_PINCHANGE;
                    pIpcSC->Header.Timeout = 0;
                    pIpcSC->Header.Len = sizeof(REQSIMPINCHANGE);
                    pIpcSC->Type = PIN;
                    strncpy((char*)pIpcSC->NewPIN, arg.pinchange.newpin, strlen(arg.pinchange.newpin));
                    strncpy((char*)pIpcSC->OldPIN, arg.pinchange.oldpin, strlen(arg.pinchange.oldpin));
            }
            break;
            case ENABLEPIN: //enablePIN
            {
                    PREQSIMPINENABLE pIpcPE = (PREQSIMPINENABLE)pIpc;
                    pIpcPE->Header.TID = SEQ_TID;
                    pIpcPE->Header.MessageID = REQ_SIM_PINENABLE;
                    pIpcPE->Header.Timeout = 0;
                    pIpcPE->Header.Len = sizeof(REQSIMPINENABLE);
                    strncpy((char*)pIpcPE->Pin, arg.pincode, strlen(arg.pincode));
            }
            break;
            case DISABLEPIN: //disablePIN
            {
                    PREQSIMPINENABLE pIpcPE = (PREQSIMPINENABLE)pIpc;
                    pIpcPE->Header.TID = SEQ_TID;
                    pIpcPE->Header.MessageID = REQ_SIM_PINDISABLE;
                    pIpcPE->Header.Timeout = 0;
                    pIpcPE->Header.Len = sizeof(REQSIMPINENABLE);
                    strncpy((char*)pIpcPE->Pin, arg.pincode, strlen(arg.pincode));
            }
            break;
            case SENDSMS: //SendSMS
            {
                    PREQSMSSEND pIpcSS = (PREQSMSSEND)pIpc;
                    pIpcSS->Header.TID = SEQ_TID;
                    pIpcSS->Header.MessageID = REQ_SMS_SEND;
                    pIpcSS->Header.Timeout = 0;
                    pIpcSS->Header.Len = sizeof(REQSMSSEND)+strlen(arg.sms_args.text);
                    pIpcSS->SMSRoute = SMS_SEND_ROUTE_CSD;
                    pIpcSS->MoreMessagesToSend = FALSE;
                    pIpcSS->RepeatedMessage = FALSE;
                    strncpy((char *)pIpcSS->ReceiverAddress, arg.sms_args.receiver_number, strlen(arg.sms_args.receiver_number)-1);
                    strncpy((char *)pIpcSS->Sms.ScAddr, arg.sms_args.sms_center_number, strlen(arg.sms_args.sms_center_number)-1);
                    
                    //supports only plain text
                    pIpcSS->Sms.Type = SMS_7BITASCII;
                    pIpcSS->Sms.Length = strlen(arg.sms_args.text);
                    strcpy((char *)pIpcSS->Sms.SmsData, arg.sms_args.text);
            }
            break;
            case PHONERESET: //reset
            {
                    PREQPHONERESET pIpcPR = (PREQPHONERESET)pIpc;
                    pIpcPR->Header.TID = SEQ_TID;
                    pIpcPR->Header.MessageID = REQ_PHONE_RESET;
                    pIpcPR->Header.Timeout = 0;
                    pIpcPR->Header.Len = sizeof(REQPHONERESET);
            }
            break;
            case RUNGSMALG: //runGSMalg
            {
                    PREQSIMRUNGSMALG pIpcRG = (PREQSIMRUNGSMALG)pIpc;
                    pIpcRG->Header.TID = SEQ_TID;
                    pIpcRG->Header.MessageID = REQ_SIM_RUNGSMALG;
                    pIpcRG->Header.Timeout = 0;
                    pIpcRG->Header.Len = sizeof(REQSIMRUNGSMALG);
                    pIpcRG->Type = GSM_ALGORITHM;
                    memset(pIpcRG->Random,0, sizeof(pIpcRG->Random));
                    if (strlen(arg.gsm_alg) == 0) {
                            if (!read_random((char *)pIpcRG->Random, MAX_RANDOM_LENGTH)) {
                                    return -1;
                            }
                    } else {
                            strncpy((char *)pIpcRG->Random, arg.gsm_alg, strlen(arg.gsm_alg)-1);
                    }
            }
            break;
            case DIALUPDISCONNECT: //dialupdisconnect
            {
                    PREQDIALUPDISCONNECT pIpcDIC = (PREQDIALUPDISCONNECT)pIpc;
                    pIpcDIC->Header.TID = SEQ_TID;
                    pIpcDIC->Header.MessageID = REQ_DIALUP_DISCONNECT;
                    pIpcDIC->Header.Timeout = 0;
                    pIpcDIC->Header.Len = sizeof(REQDIALUPDISCONNECT);
            }
            break;
            case ENABLEGSMRADIO: //enableGSMradio
            {
                    PREQNETGSMRADIOACCESS pIpcGA = (PREQNETGSMRADIOACCESS)pIpc;
                    pIpcGA->Header.TID = SEQ_TID;
                    pIpcGA->Header.MessageID = REQ_NET_ENABLE_GSMRADIOACCESS;
                    pIpcGA->Header.Timeout = 0;
                    pIpcGA->Header.Len = sizeof(REQNETGSMRADIOACCESS);
            }
            break;
            case DISABLEGSMRADIO: //disableGSMradio
            {
                    PREQNETGSMRADIOACCESS pIpcGD = (PREQNETGSMRADIOACCESS)pIpc;
                    pIpcGD->Header.TID = SEQ_TID;
                    pIpcGD->Header.MessageID = REQ_NET_DISABLE_GSMRADIOACCESS;
                    pIpcGD->Header.Timeout = 0;
                    pIpcGD->Header.Len = sizeof(REQNETGSMRADIOACCESS);
            }
            break;
            case SETSECURITYLEVEL: //setsecuritylevel
            {
                    PREQSIMSETSECURITY pIpcSS = (PREQSIMSETSECURITY)pIpc;
                    pIpcSS->Header.TID = SEQ_TID;
                    pIpcSS->Header.MessageID = REQ_SIM_SETSECURITYLEVEL;
                    pIpcSS->Header.Timeout = 0;
                    pIpcSS->Header.Len = sizeof(REQSIMSETSECURITY);
                    pIpcSS->Level = SIM_SECURITY_LEVEL_NONE;
                    strncpy((char*)pIpcSS->Code, arg.pincode, strlen(arg.pincode));
            }
            break;
            case VERIFYSECURITYCODE: //verifysecuritycode
            {
                    PREQSIMSETSECURITY pIpcSS = (PREQSIMSETSECURITY)pIpc;
                    pIpcSS->Header.TID = SEQ_TID;
                    pIpcSS->Header.MessageID = REQ_SIM_VERIFYSECURITYCODE;
                    pIpcSS->Header.Timeout = 0;
                    pIpcSS->Header.Len = sizeof(REQSIMSETSECURITY);
                    pIpcSS->Level = SIM_SECURITY_LEVEL_PHONE;
                    strncpy((char*)pIpcSS->Code, arg.pincode, strlen(arg.pincode));
            }
            break;
            case CHANGESECURITYCODE: //changesecuritycode
            {
                    PREQSIMSETSECURITY pIpcSS = (PREQSIMSETSECURITY)pIpc;
                    pIpcSS->Header.TID = SEQ_TID;
                    pIpcSS->Header.MessageID = REQ_SIM_CHANGESECURITYCODE;
                    pIpcSS->Header.Timeout = 0;
                    pIpcSS->Header.Len = sizeof(REQSIMSETSECURITY);
                    pIpcSS->Level = SIM_SECURITY_LEVEL_PHONE;
                    strncpy((char*)pIpcSS->NewCode, arg.pinchange.newpin, strlen(arg.pinchange.newpin));
                    strncpy((char*)pIpcSS->Code, arg.pinchange.oldpin, strlen(arg.pinchange.oldpin));
            }
            break;
            case VERIFYPUKCODE: //verifyPUKcode
            {
                    PREQSIMPUKVERIFY pIpcPV = (PREQSIMPUKVERIFY)pIpc;
                    pIpcPV->Header.TID = SEQ_TID;
                    pIpcPV->Header.MessageID = REQ_SIM_PUKVERIFY;
                    pIpcPV->Header.Timeout = 0;
                    pIpcPV->Header.Len = sizeof(REQSIMPUKVERIFY);
                    pIpcPV->Type = PIN;
                    strncpy((char*)pIpcPV->PUK, arg.pinchange.oldpin, strlen(arg.pinchange.oldpin));
                    strncpy((char*)pIpcPV->NewPIN, arg.pinchange.newpin, strlen(arg.pinchange.newpin));
            }
            break;
            case READSMS: //readSMS
            {
                    PREQSMSREAD pIpcRS=(PREQSMSREAD)pIpc;
                    pIpcRS->Header.TID = SEQ_TID;
                    pIpcRS->Header.MessageID = REQ_SMS_READ;
                    pIpcRS->Header.Timeout = 0;
                    pIpcRS->Header.Len = sizeof(REQSMSREAD);
                    pIpcRS->Index=arg.smsindex;
            }
            break;
            case SMSENUM: //SMSenum
            {
                    PREQSMSENUM pIpcSE=(PREQSMSENUM)pIpc;
                    pIpcSE->Header.TID = SEQ_TID;
                    pIpcSE->Header.MessageID = REQ_SMS_ENUM;
                    pIpcSE->Header.Timeout = 0;
                    pIpcSE->Header.Len = sizeof(REQSMSENUM);
            }
            break;
            case SMSPROCESSED: //SMSprocessed
            {
                    PREQSMSPROCESSED pIpcSP=(PREQSMSPROCESSED)pIpc;
                    pIpcSP->Header.TID = SEQ_TID;
                    pIpcSP->Header.MessageID = REQ_SMS_PROCESSED;
                    pIpcSP->Header.Timeout = 0;
                    pIpcSP->Header.Len = sizeof(REQSMSPROCESSED);
                    pIpcSP->Index=arg.smsindex;
            }
            break;
            default:
                    DBG_ERROR ("got unknown cmd\n");
                    return -1;
        }
        
        if ( (len = send_message(pIpc, pIpc_rsp)) < 0){
                DBG_ERROR("send_message failed\n");
                return -1;
        }

        switch (command){
             case READSMS: //readSMS
             {
                     PRSPSMSREAD pIpcRS=(PRSPSMSREAD)pIpc_rsp;
                     unsigned int smslen, datalen = len - sizeof(IPCRSPHEADER);
                     if (datalen > sizeof(SMS)){
		             printf("Sender: %s\n", pIpcRS->Sms.SenderAddr);
                             smslen = pIpcRS->Sms.Length;
                             if ((smslen+sizeof(SMS)-1) > datalen)
                                     printf("truncated SMS\n");
                             else {
				     printf("%s\n", pIpcRS->Sms.SmsData);
                             }                                     
                     }
                     else
                             printf("Truncated SMS\n");
             }
             break;
             case SMSENUM: //SMSenum
             {
                     PRSPSMSENUM pIpcSE=(PRSPSMSENUM)pIpc_rsp;
                     printf("New SMS %i\n", pIpcSE->NumNew);
                     printf("Processed SMS %i\n", pIpcSE->NumProcessed);
                     printf("Free SMS %i\n", pIpcSE->NumEmpty);
             }
             break;
        }

        return 0;
}

