/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

#include <gsmbase.h>
#include <profilemib.h>

#ifndef NOKIA_CTL_H
#define NOKIA_CTL_H

#define CONF_FILE_NAME "/etc/pcmcia/nokia_cs.opts"

#define NEW_KEY_SHARED 1
#define NEW_KEY_PERSONAL 2

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE  1
#endif

/* Command defines
   These define the commands given from the UI to the engine
*/
#define ENABLE_WLAN     0
#define DISABLE_WLAN    1
#define ENABLE_GSM      2
//MIBCOMMANDS
#define NETNAMES        10
#define CASE            11
#define OPMODE          12
#define SECPOLICY       13
#define SECMODE         14
#define SECSHAREDKEYS   15
#define MINCHANNEL      16
#define MAXCHANNEL      17
#define RESTRICTCHANNEL 18
#define SECSTATIONKEY   19
//GPRS or GSM commands
#define CHANGEPIN       30
#define ENABLEPIN       31
#define DISABLEPIN      32
#define SENDSMS         33
#define PHONERESET      37
#define RUNGSMALG          40
#define DIALUPDISCONNECT   43
#define ENABLEGSMRADIO     44
#define DISABLEGSMRADIO    45
#define SETSECURITYLEVEL   48
#define VERIFYSECURITYCODE 49
#define CHANGESECURITYCODE 50
#define VERIFYPUKCODE      51
#define READSMS            53
#define SMSENUM            54
#define SMSPROCESSED       55

typedef struct wep_args
{
        int active;                //Active key
	unsigned char keylen[4];	// Key lengths
	unsigned char keydata[64];	// Data of keys
        char netname[33];       // 32 + 1 for terminating null
} wep_args_t;

typedef struct pinchange
{
        char oldpin[MAX_PIN_LENGTH+1];
        char newpin[MAX_PIN_LENGTH+1];
} pinchange_t;

typedef struct sms_args
{
        char receiver_number[MAX_GSM_NUMBER_LENGTH+1];
        char sms_center_number[MAX_GSM_NUMBER_LENGTH+1];
        char text[2048];
} sms_args_t;

typedef struct dialup_args
{
        char validitymask;
        long int modem_bitrate;
        char modem_connection_method;
        long int gprs_ppp_timeout;
        char gprs_AP_name[MAX_GPRS_AP_NAME_LENGTH+1];
} dialup_args_t;

// This union holds arguments for parse_main_args()
union args {
        char netname[33];       // 32 + 1 for terminating null
        int casesensitive;
        int opmode; //operating mode
        int minchannel;
        int maxchannel;
        int wepmode;
        int weppolicy; //station or network
        int restrictchannel;
        char pincode[MAX_PIN_LENGTH+1];
        pinchange_t pinchange;
        wep_args_t wep_args;
        sms_args_t sms_args;
        char gsm_alg[MAX_RANDOM_LENGTH+1];
        int smsindex;
};

extern char* mib_subcmds_options[];

int getcmddata(char *filename,char *cmd, char* x);
unsigned char t_strcasecmp(char *a, char *b);
char *parse_tag(char *s,char *tag, char *tmp);
char *parse_command_args(unsigned char *data);
int parse_main_args (int command, union args arg);
unsigned char hexstrtochar(char *str);
int handle_key (char *buf, char *src);
void ask_pincode(char* pincode, char* prompt);
char* read_random (char* buf, int length);


#endif





