/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

extern "C" {
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
}

#include "nokia_ctl.h"
#include "sndmsg.h"

// Ask pincode from the user
void ask_pincode(char *pincode, char* prompt)
{
        char* code;
        if (!pincode) return;
        code = getpass(prompt);
        if (code == NULL) {
                printf("getpass() failed\n");
                exit(-1);
        }
        //DBG("%s\n",pass);
        strncpy(pincode,code,strlen(code));
}

// Read a random number into a buffer
char* read_random (char* buf, int length)
{
        FILE *f = NULL;
        
        if (!buf)
                return NULL;
        
        f = fopen("/dev/urandom","rb");
        if (!f) {
                printf("Could not open /dev/urandom\n");
                return NULL;
        }
        if (!fgets(buf,length,f)) {
                printf("Could not read /dev/urandom\n");
                return NULL;
        }       
        fclose(f);
        
        return buf;
}
        
/* convert a string of hexvalue to a UINT8 */
unsigned char hexstrtochar(char *str)
{
        unsigned char x, y;
        int n = strlen(str);
        
        if(n < 1) return 0;
        
        x = str[0];
        if(x >= '0' && x <= '9') x = x - '0';
        if(x >= 'a' && x <= 'f') x = x - 'a' + 10;
        if(x >= 'A' && x <= 'F') x = x - 'A' + 10;
        
        if(n < 2) {
                if(x > 15) return 0;
                return x;
        }
        
        y = str[1];
        if(y >= '0' && y <= '9') y = y - '0';
        if(y >= 'a' && y <= 'f') y = y - 'a' + 10;
        if(y >= 'A' && y <= 'F') y = y - 'A' + 10;
        
        return x*16+y;
}

void print_error_subcmds (int index)
{
        int i=0;
        for (i = 0; strcmp(mib_subcmds_options[i], "endlist"); i++) {
        }
        if (index < 0 || index >= i) {
                printf("Invalid index was given for error printing\n");
                exit (-1);
        }       
        printf("Argument was missing or invalid, options: %s\n", mib_subcmds_options[index]);
        exit (-1);
}

int getcmddata(char *filename,char *cmd, char* x)
{
    FILE *f = NULL;
    char *z = NULL;
    int i,l;
    
    if (!cmd || !x) return -1;
    
    f = fopen(filename,"rb");
    if(!f) {
        printf("nokia_ctl: Unable to open %s\n",filename);
        goto fail;
    }

    // Find the cmd

    while(!feof(f)) {
        z = fgets(x,256,f);
        if(!z) {
            printf("nokia_ctl: Unknown command: %s\n", cmd);
            goto fail;
        }

        if(x[0] != '[') continue;

        if(!strncmp(&x[1], cmd, strlen(cmd))) {
                if (x[strlen(cmd)+1] == ']') //Check also for the lengths
                        break;
        }
    }
    if(feof(f)) {
        printf("nokia_ctl: Unknown Command.\n");
        goto fail;
    }

    z = x;
    while(!feof(f)) {

        if(fgets(z, 256, f) == NULL) break; 

        if(z[0] == '*' || z[0] < 32) break;

        l = strlen(z);

        // Remove suspicious characters

        for(i=0;i < l;i++) {
            if(z[i] < 32) {
                z[i] = '\n';
                z[i+1] = '\0';
            }
        }

        z = z+strlen(z);
    }

    fclose(f);
    
    return 0;

  fail:

    if(f) fclose(f);

    return -1;
}

char* parse_command_args(char *data)
{
        char *cmd = NULL, *ar = NULL;
        int i, len = strlen(data);

        if(!data) return NULL;

        // Read till [ char

        for(i=0;i < len;i++) if(data[i] == '[') break;

        cmd = &data[i+1];

        for(;i < len;i++) {
                if(data[i] == ']') {
                        data[i] = '\0';
                        ++i;
                        ar = &data[i];
                        break;
                }
        }
    
        DBG("cmd:\n%s\n",data);
    
        if(!ar) {
                printf("Character device command parse error.\n");
                DBG_ERROR("parselen=%d\n",len);
                DBG_ERROR("cmd:\n%s\n",data);
                return NULL;
        }
    
        // Now cmd = Command string and ar = arguments
        // parsed by parse_tag
        return ar;
}

char *parse_tag(char *s, char *tag, char *tmp)
{
        char *z;
        int i;

        if (!s || !tag || !tmp) return NULL;
        
        //DBG("Parse tag:%s:(1stchar=%d)\n",tag,(unsigned int)s[0]);
        
        z = s;
    
        for(;;) {
                for(;;) {
                        z = strstr(z,tag);
                        if(!z) return NULL;
                                                
                        // If the s is like:
                        // use_ssid=xxxx
                        // ssid=xxxx
                        // This would only find the use_ssid with ssid tag...
                        // Which should of cource not happen. Therefore, we check
                        // The prev chracter. If its \n or space or something it is 
                        // ok. Otherwise continue strstr...
	  
                        if(z == s) break; // Don't segfault 
                        
                        if(z[-1] < 33) break;
                        
                        z = z+1; // Don't find the same again
                }
                
                z = z+strlen(tag);
                while(*z == ' ') z = z+1;
                if(*z++ != '=') continue;
                for(i=0;*z != '\n' && *z != '\0';i++) {
                        tmp[i] = *z++;
                }
                tmp[i] = '\0';
                return(tmp);
        }
}

unsigned char t_strcasecmp(char *a, char *b)
{
        if(!a||!b) return 255;
        
        return(strcmp(a,b));
}

int handle_key (char *buf, char *src)
{
        int length,i;

        if (!buf || !src) return -1;
        
        if (strlen(src) == 0) { //empty key
                return 0;
        }
        
        /* Check if the key is in string or hex format */
        if(!strncmp(src,"0x",2)) {
                length = strlen(src)/2-1;
                /* Length must be either 40, 104 or 128 bits (+IV) */
                if (length != 5 && length != 13 && length != 16) {
                        printf("Invalid key length in one of your WEP keys.\n");
                        return -1;
                }
                src = src+2;
                for (i=0;i < length;i++) {
                        buf[i] = hexstrtochar(src);
                        src = src+2;
                }
        } else {
                length = strlen(src);
                /* Length must be either 40, 104 or 128 bits (+IV) */
                if (length != 5 && length != 13 && length != 16) {
                        printf("Invalid key length in one of your WEP keys.\n");
                        return -1;
                }
                memcpy(buf, src, length);
        }
        return length;
}
