/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/signal.h>
}
#include <ipcbase.h>
#include <wlanctlreq.h>
#include <wlanmibreq.h>
#include <mibreq.h>
#include <wlanreq.h>
//GPRS
#include <sim.h>
#include <sms.h>
#include <net.h>
#include <phone.h>
#include <dialup.h>
#include <contacts.h>
//local
#include "sndmsg.h"
#include "nokia_ctl.h"

#define WLAN_MODE 1
#define GSM_MODE  2

#define CMD 1

//Local function declarations
void print_error_subcmds (int index);

static char* cmds[] = {
        "enable_wlan",
        "disable_wlan",
        "wlan_mibset",
        "wlan_mibget",
        "profile",
        "enable_gsm",
        "disable_gsm",
        "gsm",
        "endlist"  //always last entry
};

static char* gsm_subcmds[] = {
        "changePIN",
        "enablePIN",
        "disablePIN",
        "sendSMS",
        "runGSMalg",
        "enableGSMradio",
        "disableGSMradio",
        "setsecuritylevel",
        "verifysecuritycode",
        "changesecuritycode",
        "phonereset",
        "verifyPUK",
        "readSMS",
        "SMSenum",
        "SMSprocessed",
        "endlist"  //always last entry
};

static char* mib_subcmds[] = {
        "netnames",
        "case",
        "opmode",
        "secpolicy",
        "secmode",
        "secsharedkeys",
        "minchannel",
        "maxchannel",
        "restrictchannel",
        "secpersonalkey",
        "endlist"  //always last entry
};

char* mib_subcmds_options[] = {
        "<network name>",
        "sensitive insensitive",
        "infra adhoc",
        "station network",
        "open shared",
        "NA",
        "<channel number>",
        "<channel number>",
        "<channel number>",
        "NA",
        "endlist"  //always last entry
};

//signal handler for CTRL-C
void ouch (int sig)
{
        DBG ("OUCH! - I got signal %d\n", sig);
        return;
}

void subcmd_usage(char* name, int index, char** subcmds){
        int i;
        printf("usage: %s %s (", name, cmds[index]);
        for (i = 0; strcmp(subcmds[i], "endlist"); i++)
                printf("%s/", subcmds[i]);
        printf(")\n");
        return;
}

/* main() parses the arguments and sends them to parse_main_args() which is
   the engine */
int main (int argc, char* argv[]){
        static int SEQ_TID = 1;
        static char buffer[BUF_SIZE];
        PIPCMESSAGE pIpc = (PIPCMESSAGE) buffer;
        PIPCRSP pIpc_rsp = (PIPCRSP) buffer;
        int i, len, index, subindex, length, newkeys = 0, ret = 0;
        int mode = 0;
        char* q = NULL;
        char buf[128];
        union args arg;
        unsigned char wepbuf[16]; //for WEP keys
        static char netname_buf[25]; //buffer for netname
        char tmp_buf[8192]; // For getcmddata()
        int active_key = -1;
        int opmode = -1;
        
        bzero(buffer, BUF_SIZE);
        bzero(wepbuf, sizeof(wepbuf));
        bzero(netname_buf, sizeof(netname_buf));
        bzero(tmp_buf, sizeof(tmp_buf));
        bzero(&arg, sizeof(union args));

        (void) signal(SIGINT, ouch);

        if (argc == 1){
                printf("usage: %s cmd (", argv[0]);
                for (i = 0; strcmp(cmds[i], "endlist"); i++)
                        printf("%s/", cmds[i]);
                printf(")\n");
                return -1;
        }

        //check which command
        for (index = 0; strcmp(cmds[index], argv[CMD]); index++){
                if(!strcmp(cmds[index], "endlist")){
                        printf("Unknown command\n");
                        return -1;
                }
        }
        switch (index) {
            case 0: //enable_wlan
                    ret = parse_main_args(0, arg);
                    break;
            case 1: //disable_wlan
                    ret = parse_main_args(1, arg);
                    break;
            case 2: //mibset
            {
                    if (argc <= 2){
                            printf("No subcommand\n");
                            subcmd_usage(argv[0], index, mib_subcmds);
                            return -1;
                    }
                    for (subindex = 0; strcmp(mib_subcmds[subindex], argv[2]); subindex++){
                            if(!strcmp(mib_subcmds[subindex], "endlist")){
                                    printf("Unknown subcommand %s\n", argv[2]);
                                    subcmd_usage(argv[0], index, mib_subcmds);
                                    return -1;
                            }
                    }

                    if (argc < 4) {
                            print_error_subcmds(subindex);
                    }
                    
                    DBG("subcmd: %s subindex %d\n", argv[2], subindex);
                    
                    switch(subindex){
                        case 0: //netnames
                                if (strlen(argv[3]) > 32) {
                                        printf("Too long SSID!\n");
                                        return -1;
                                }
                                strncpy(arg.netname, argv[3], strlen(argv[3]));
                                ret = parse_main_args(NETNAMES, arg);
                                break;
                        case 1: //case
                                if (!strcmp(argv[3], "sensitive")) {
                                        arg.casesensitive = 0;
                                } else if (!strcmp(argv[3], "insensitive")) {
                                        arg.casesensitive = 1;
                                } else {
                                        print_error_subcmds(index);
                                }
                                arg.casesensitive = atoi(argv[3]);
                                ret = parse_main_args(CASE, arg);
                                break;
                        case 2: //opmode
                                if (!strcmp(argv[3], "infra")) {
                                        arg.opmode = 1;
                                } else if (!strcmp(argv[3], "adhoc")) {
                                        arg.opmode = 0;
                                } else {
                                        print_error_subcmds(index);
                                }
                                ret = parse_main_args(OPMODE, arg);
                                break;
                        case 3: //secpolicy
                                if (!strcmp(argv[3], "network")) {
                                        arg.weppolicy = WEPMODE_POLICY_NETWORK;
                                } else if (!strcmp(argv[3], "station")) {
                                        arg.weppolicy = WEPMODE_POLICY_STATION;
                                } else {
                                        print_error_subcmds(index);
                                }
                                ret = parse_main_args(SECPOLICY, arg);
                                break;
                        case 4: //secmode
                                if (!strcmp(argv[3],"open")) {
                                        arg.wepmode = WEPMODE_OPEN;
                                } else if (!strcmp(argv[3], "shared")) {
                                        arg.weppolicy = WEPMODE_AUTHSHARED;
                                } else {
                                        print_error_subcmds(subindex);
                                }
                                ret = parse_main_args(SECMODE, arg);
                                break;
                        case 5: //secsharedkeys
                                printf("Not implemented, use a profile\n");
                                return -1;
                                break;
                        case 6: //minchannel
                                if(atoi(argv[3]) > 13 || atoi(argv[3]) < 1) {
                                        printf("Invalid minimum channel %d\n",atoi(argv[3]));
                                        return -1;
                                } else {
                                        arg.minchannel = atoi(argv[3]);
                                        ret = parse_main_args(MINCHANNEL, arg);
                                }
                                break;
                        case 7: //maxchannel
                                if(atoi(argv[3]) > 13 || atoi(argv[3]) < 1) {
                                        printf("Invalid maximum channel %d\n",atoi(argv[3]));
                                        return -1;
                                } else {
                                        arg.maxchannel = atoi(argv[3]);
                                        ret = parse_main_args(MAXCHANNEL, arg);
                                }
                                break;
                        case 8: //restrictchannel
                                if(atoi(argv[3]) > 13 || atoi(argv[3]) < 0) {
                                        printf("Invalid channel %d for restriction\n",atoi(argv[3]));
                                        return -1;
                                } else {
                                        arg.restrictchannel = atoi(argv[3]);
                                        ret = parse_main_args(RESTRICTCHANNEL, arg);
                                }
                                break;
                        case 9: //secstationkey
                                printf("Not implemented, use a profile\n");
                                return -1;
                                break;
                        default:
                                printf("Subcommand not implemented\n");
                                return -1;
                    }
            }
            break;
            case 3: //mibget
            {
                    PREQMIBGET pIpcMG = (PREQMIBGET)pIpc;
                    
                    if (argc <= 2){
                            printf("No subcommand\n");
                            subcmd_usage(argv[0], index, mib_subcmds);
                            return -1;
                    }
                    
                    for (i = 2; i < argc; i++){
                            for (subindex = 0; strcmp(mib_subcmds[subindex], argv[i]); subindex++){
                                    if(!strcmp(mib_subcmds[subindex], "endlist")){
                                            printf("Unknown subcommand %s\n", argv[i]);
                                            subcmd_usage(argv[0], index, mib_subcmds);
                                            return -1;
                                    }
                            }
                            
                            DBG("subcmd: %s\n", argv[i]);
                            
                            pIpcMG->Header.TID = SEQ_TID;
                            pIpcMG->Header.MessageID = REQ_WLANMIB_GET;
                            pIpcMG->Header.Timeout = 0;
                            pIpcMG->Header.Len = sizeof(REQMIBGET);
                            
                            switch(subindex){
                                case 0: //netnames
                                        pIpcMG->Parameter = MIB_WLAN_SET_NETNAMES;
                                        break;
                                case 1: //case
                                        pIpcMG->Parameter = MIB_WLAN_SET_CASESENSITIVE;
                                        break;
                                case 2: //opmode
                                        pIpcMG->Parameter = MIB_WLAN_SET_OPMODE;
                                        break;
                                case 3: //secpolicy
                                        pIpcMG->Parameter = MIB_WLAN_SEC_POLICY;
                                        break;
                                case 4: //secmode
                                        pIpcMG->Parameter = MIB_WLAN_SEC_MODE;
                                        break;
                                case 5: //secsharedkeys
                                        pIpcMG->Parameter = MIB_WLAN_SEC_SHAREDKEYS;
                                        break;
                                case 6: //minchannel
                                        pIpcMG->Parameter = MIB_WLAN_SET_MINCHANNEL;
                                        break;
                                case 7: //maxchannel
                                        pIpcMG->Parameter = MIB_WLAN_SET_MAXCHANNEL;
                                        break;
                                case 8: //restrictchannel
                                        pIpcMG->Parameter = MIB_WLAN_SET_RESTRICTCHANNEL;
                                        break;
                                case 9: //secstationkey
                                        pIpcMG->Parameter = MIB_WLAN_SEC_STATIONKEY;
                                        break;
                                default:
                                        printf("Subcommand not implemented\n");
                                        return -1;
                            }
                            if ( (len = send_message(pIpc, pIpc_rsp)) < 0){
                                    DBG_ERROR("send_message failed\n");
                                    return -1;
                            }
                    }
            }
            break;
            case 4: //profile
            {
                    if (argc < 3) {
                            printf("No profile name was given\n");
                            return -1;
                    }       
                    if (getcmddata(CONF_FILE_NAME, argv[2], tmp_buf) < 0 || !tmp_buf) {
                            printf("Command Failed.\n");
                            return -1;
                    }
                    DBG("Command:\n%s\n",tmp_buf);
                    
                    // WLAN settings start
                    // First set minchannel and maxchannel
                    arg.minchannel = 1;
                    ret = parse_main_args(MINCHANNEL, arg);
                    arg.maxchannel = 13;
                    ret = parse_main_args(MAXCHANNEL, arg);

                    if((q = parse_tag(tmp_buf, "restrictchannel", buf))) {
                            if(atoi(q) > 13 || atoi(q) < 0) {
                                    printf("Invalid channel %d for restriction\n", atoi(q));
                                    return -1;
                            }
                            // Empty channel is also = 0
                            arg.restrictchannel = atoi(q);
                            ret = parse_main_args(RESTRICTCHANNEL, arg);
                    } else {
                            arg.restrictchannel = 0;
                            ret = parse_main_args(RESTRICTCHANNEL, arg);
                    }
                    if((q = parse_tag(tmp_buf, "mode", buf))) {
                            mode = WLAN_MODE; //to distinguish from gsm
                            if(!t_strcasecmp(q, "infra")) {
                                    arg.opmode = 1;
                                    ret = parse_main_args(OPMODE, arg);
                            } else if(!t_strcasecmp(q, "adhoc")) {
                                    arg.opmode = 0;
                                    ret = parse_main_args(OPMODE, arg);
                            } else {
                                    printf("Unknown mode!\n");
                                    return -1;
                            }
                            opmode = arg.opmode;
                            
                    } else {
                            printf("mode not set in profile\n");
                            return -1;
                    }
                    if ((q = parse_tag(tmp_buf,"ssid", buf))) {
                            if (strlen(q) > 32) {
                                    printf("ssid should be shorter than 33 characters\n");
                                    return -1;
                            } else {
                                    strncpy(arg.netname, q, strlen(q));
                                    //Keep the name in memory for authentication data
                                    strncpy(netname_buf, q, strlen(q));
                                    ret = parse_main_args(NETNAMES, arg);
                            }
                    } else {
                            printf("ssid not set in profile\n");
                            return -1;
                    }
                    if((q = parse_tag(tmp_buf,"station_key", buf))) {
                            length = handle_key((char*)&wepbuf,q);
                            if (length < 0) {
                                    printf("Error in station key\n");
                                    return -1;
                            } else if (length != 0) {
                                    memset(&arg.wep_args, 0, sizeof(arg.wep_args));
                                    // We'll use the same buffer for personal and shared keys but we'll have to send the message immediately
                                    arg.wep_args.keylen[0] = length;
                                    memcpy(&arg.wep_args.keydata[0], wepbuf, length); 
                                    DBG("Setting personal keys\n");
                                    ret = parse_main_args(SECSTATIONKEY, arg);
                                    
                                    newkeys |=NEW_KEY_PERSONAL;
                            }
                    } else {
                            memset(&arg.wep_args, 0, sizeof(arg.wep_args));
                            parse_main_args(SECSTATIONKEY, arg);
                    }
                    if((q = parse_tag(tmp_buf,"active_key", buf))) {
                            if (strlen(q) != 0) {
                                    active_key = atoi(q);
                                    if (active_key < 0 || active_key > 4){
                                            printf("Error in active_key\n");
                                            return -1;
                                    } else {
                                            DBG("Active key set to: %d\n", active_key);
                                            memset(&arg.wep_args, 0, sizeof(arg.wep_args));
                                    }
                            }       
                    }
                    if (active_key != 0){
                            if((q = parse_tag(tmp_buf,"shared_key1", buf))) {
                                    length=handle_key((char*)&wepbuf,q);
                                    if (length < 0) {
                                            printf("Error in shared_key1\n");
                                            return -1;
                                    } else if (length != 0) {
                                            arg.wep_args.keylen[0] = length;
                                            memcpy(&arg.wep_args.keydata[0], wepbuf, length);
                                            newkeys |= NEW_KEY_SHARED;
                                            if (active_key == -1){
                                                    active_key = 1;
                                                    DBG("Active key set to: %d\n", active_key);
                                            }
                                    }
                            }
                            if((q = parse_tag(tmp_buf,"shared_key2", buf))) {
                                    length = handle_key((char*)&wepbuf,q);
                                    if (length < 0) {
                                            printf("Error in shared_key2\n");
                                            return -1;
                                    } else if (length != 0) {
                                            arg.wep_args.keylen[1] = length;
                                            memcpy(&arg.wep_args.keydata[16], wepbuf, length);
                                            newkeys |= NEW_KEY_SHARED;
                                    }
                            }
                            if((q = parse_tag(tmp_buf,"shared_key3", buf))) {
                                    length = handle_key((char*)&wepbuf,q);
                                    if (length < 0) {
                                            printf("Error in shared_key3\n");
                                            return -1;
                                    } else if (length != 0) {
                                            arg.wep_args.keylen[2] = length;
                                            memcpy(&arg.wep_args.keydata[32], wepbuf, length);
                                            newkeys |= NEW_KEY_SHARED;
                                    }
                            }
                            if((q = parse_tag(tmp_buf,"shared_key4", buf))) {
                                    length = handle_key((char*)&wepbuf,q);
                                    if (length < 0) {
                                            printf("Error in shared_key4\n");
                                            return -1;
                                    } else if (length != 0) {
                                            arg.wep_args.keylen[3] = length;
                                            memcpy(&arg.wep_args.keydata[48], wepbuf, length);
                                            newkeys |= NEW_KEY_SHARED;
                                    }
                            }
                            if (active_key == -1){
                                    if (newkeys & NEW_KEY_SHARED){
                                            printf("Error in active_key, shared keys exists and active_key is not set\n");
                                            return -1;
                                    }
                                    else
                                            DBG("active_key not set\n");
                            }
                            else if (arg.wep_args.keylen[active_key-1] == 0){
                                    printf("Error in active_key, shared_key%i not set\n", active_key);
                                    return -1;
                            } else {
                                    arg.wep_args.active = active_key-1;
                            }
                    }
                    if (mode == WLAN_MODE) {
                            if (newkeys & NEW_KEY_SHARED) {
                                    //Set the shared keys
                                    DBG("Setting shared keys\n");
                                    strncpy(arg.wep_args.netname,netname_buf, strlen(netname_buf));
                                    ret = parse_main_args(SECSHAREDKEYS, arg);
                            } else {
                                    //Unset the shared keys 
                                    memset(&arg.wep_args, 0, sizeof(arg.wep_args));
                                    ret = parse_main_args(SECSHAREDKEYS, arg);
                            }
                            
                            if (newkeys) {
                                    if (opmode != 1){
                                            printf("WEP is not supported in AdHoc mode (please set active_key=0)\n");
                                            return -1;
                                    }

                                    //WEP policy
                                    arg.weppolicy = WEPMODE_POLICY_NETWORK;
                                    ret = parse_main_args(SECPOLICY, arg);

                                    //WEP mode
                                    arg.wepmode = 0;
                                    if (newkeys & NEW_KEY_SHARED) {
                                            printf ("Enable Shared Key authentication (shared_key%i)\n", active_key);
                                            arg.wepmode |= WEPMODE_USESHARED;
                                    }
                                    if (newkeys & NEW_KEY_PERSONAL) {
                                            printf ("Enable Personal Key authentication\n");
                                            arg.wepmode |= WEPMODE_USEPERSONAL;
                                    }
                            } else { //Open system auth
                                    printf ("Enable Open System authentication\n");
                                    arg.wepmode = WEPMODE_OPEN;
                            }
                            ret = parse_main_args(SECMODE, arg);
                            
                            // If we get this far then enable wlan
                            ret = parse_main_args(ENABLE_WLAN, arg);
                    }
                    // WLAN settings stop
            }
            break;
            case 5: //enable_gsm
                    printf("Please give the PIN code\n");
                    ask_pincode(arg.pincode, "PIN:");
                    ret = parse_main_args(ENABLE_GSM, arg);
                    break;
            case 6: //disable_gsm
                    //parse_main_args(DIALUPDISCONNECT, arg);
                    ret = parse_main_args(DISABLEGSMRADIO, arg);
                    break;
            case 7: //gprs
                    if (argc <= 2){
                            printf("No subcommand\n");
                            subcmd_usage(argv[0], index, gsm_subcmds);
                            return -1;
                    }
                    
                    for (i = 2; i < argc; i++){
                            for (subindex = 0; strcmp(gsm_subcmds[subindex], argv[i]); subindex++){
                                    if(!strcmp(gsm_subcmds[subindex], "endlist")){
                                            printf("Unknown subcommand %s\n", argv[i]);
                                            subcmd_usage(argv[0], index, gsm_subcmds);
                                            return -1;
                                    }
                            }
                    }
                    switch (subindex) {
                        case 0: //changePIN
                                printf("Please give the old PIN code\n");
                                ask_pincode(arg.pinchange.oldpin, "PIN:");
                                printf("Please give a new PIN code\n");
                                ask_pincode(arg.pinchange.newpin, "PIN:");
                                ret = parse_main_args(CHANGEPIN, arg);
                                break;
                        case 1: //enablePIN
                                printf("Please give the PIN code\n");
                                ask_pincode(arg.pincode, "PIN:");
                                ret = parse_main_args(ENABLEPIN, arg);
                                break;
                        case 2: //disablePIN
                                printf("Please give the PIN code\n");
                                ask_pincode(arg.pincode, "PIN:");
                                ret = parse_main_args(DISABLEPIN, arg);
                                break;
                        case 3: //sendSMS
                                printf("Please type the message (max 160 chars)\n");
                                
                                fgets(arg.sms_args.text, sizeof(arg.sms_args.text),stdin);
                                printf("Please type the receiver number\n");
                                fgets(arg.sms_args.receiver_number, sizeof(arg.sms_args.receiver_number), stdin);
                                printf("Please type the SMS center number\n");
                                fgets(arg.sms_args.sms_center_number, sizeof(arg.sms_args.sms_center_number), stdin);
                                
                                ret = parse_main_args(SENDSMS, arg);
                                break;
                        case 4: //runGSMalg
                                bzero(buf, sizeof(buf));
                                printf("Type the input for gsm algorithm or \npress enter for system provided random number\n");
                                fgets(buf, sizeof(buf), stdin);
                                if (strlen(buf) > 1) {
                                        strncpy(arg.gsm_alg, buf, strlen(buf));
                                }
                                ret = parse_main_args(RUNGSMALG, arg);
                                break;
                        case 5: //enableGSMradio
                                ret = parse_main_args(ENABLEGSMRADIO, arg);
                                break;
                        case 6: //disableGSMradio
                                ret = parse_main_args(DISABLEGSMRADIO, arg);
                                break;
                        case 7: //setsecuritylevel
                                printf("Please give the security code\n");
                                ask_pincode(arg.pincode, "PIN:");
                                ret = parse_main_args(SETSECURITYLEVEL, arg);
                                break;
                        case 8: //verifysecuritycode
                                printf("Please give the security code\n");
                                ask_pincode(arg.pincode, "PIN:");
                                ret = parse_main_args(VERIFYSECURITYCODE, arg);
                                break;
                        case 9: //changesecuritycode
                                printf("Please give the current security code\n");
                                ask_pincode(arg.pinchange.oldpin, "PIN:");
                                printf("Please give a new security code\n");
                                ask_pincode(arg.pinchange.newpin, "PIN:");
                                ret = parse_main_args(CHANGESECURITYCODE, arg);
                                break;
                        case 10: //reset
                                ret = parse_main_args(PHONERESET, arg);
                                break;
                        case 11: //verifyPUK
                                printf("Please give the PUK code\n");
                                ask_pincode(arg.pinchange.oldpin, "PUK:");
                                printf("Please give a new PIN code\n");
                                ask_pincode(arg.pinchange.newpin, "PIN:");
                                ret = parse_main_args(VERIFYPUKCODE, arg);
                                break;
                        case 12: //readSMS
                                bzero(buf,sizeof(buf));
                                printf("Please type the message index\n");
                                fgets(buf,sizeof(buf),stdin);
                                if (atoi(buf) >= 0) {
                                        arg.smsindex=atoi(buf);
                                        parse_main_args(READSMS, arg);
                                }
                                break;
                        case 13: //SMSenum
                                parse_main_args(SMSENUM, arg);
                                break;
                        case 14: //SMSprocessed
                                bzero(buf,sizeof(buf));
                                printf("Please type the message index\n");
                                fgets(buf,sizeof(buf),stdin);
                                if (atoi(buf) >= 0) {
                                        arg.smsindex=atoi(buf);
                                        parse_main_args(SMSPROCESSED, arg);
                                }
                                break;
                        default:
                                printf("Subcommand not implemented\n");
                                return -1;
                    }
                    break;
            default:
                    DBG_ERROR ("got unknown cmd\n");
                    return -1;
        }
            
        SEQ_TID++;
        return ret;
}

























