/***************************************************************************
 * The contents of this file are subject to the NOKOS License Version 1.0a
 * (the "License"); you may not use this file except in compliance with the
 * License. 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF  ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The Original Software is Nokia D211 Software for Linux. 
 *
 * Copyright (c) Nokia 2002 and others. All Rights Reserved.
 *
 * Nokia is a registered trademark of Nokia Corporation.
 * Other product and company names mentioned herein may be
 * trademarks or tradenames of their respective owners. 
 *
 * Contributor(s): ______________________________________. 
 *
 **************************************************************************/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/signal.h>
#include <sys/time.h>
}

#include <ipcbase.h>
#include <ipcserv.h>
#include <ipclinux.h>
#include "sndmsg.h"

                
//signal handler for send timeout
static void sndtimeout (int sig)
{
        DBG_LOG ("Timeout in IPC message sending\n");
        (void) signal(SIGALRM, SIG_DFL);        
        return;
}

int send_message(PIPCMESSAGE pIpc, PIPCRSP pIpc_rsp){
        int TID = 0;
        int ret = 0;
        int req_key;
        int req_msqid;
        int rsp_msqid;
        int msg_len;
        int rsp_key;
        MSQMSG* pmsg;
        struct timeval tv;
        struct itimerval it;

        
        //sanity
        if (!pIpc || !pIpc_rsp){
                DBG_ERROR("null arguments, pIpc: %p, pIpc_rsp: %p\n", pIpc, pIpc_rsp);
                return -1;
        }        

        msg_len = pIpc->Header.Len; 
        //key for responce msg queue
        rsp_key = KEY_OF_MSQ(pIpc->Header.MessageID | IPC_RSP_BASE);
        //get rsp msg queue
        rsp_msqid = msgget(rsp_key, 0666 | IPC_CREAT); 
        if (rsp_msqid == -1){
                perror("msgget");
                DBG_ERROR("msgget failed for responce queue\n");
                return -1;
        }        

	if (pIpc->Header.Timeout == 0)
		pIpc->Header.Timeout = 5000;
        
        req_key = KEY_OF_MSQ(pIpc->Header.MessageID); //key for msg queue
        req_msqid = msgget(req_key, 0666 | IPC_CREAT);
        
        if (req_msqid == -1){
                perror("msgget");
                DBG_ERROR("msgget failed for request queue");
                return -1;
        }
        TID = pIpc->Header.TID;

        pmsg = (MSQMSG*)new char[msg_len + sizeof(long int)];

        if(!pmsg){
                perror("malloc");
                return -1;
        }
        memset(pmsg, 0, msg_len + sizeof(long int));
        pmsg->mtype = REQ_MSG_TYPE(pIpc->Header.MessageID);
        memcpy(pmsg->ipc_msg, pIpc, msg_len);

        DBG_TRACE("sending message to msg queue %i, key 0x%08lx\n", req_msqid, req_key);
        DBG("send request: \n");
        DBG("TID: %x\n", pIpc->Header.TID);
        DBG("Message ID: %x\n", pIpc->Header.MessageID);
        DBG("Timeout: %li\n", pIpc->Header.Timeout);
        DBG("Len: %li\n\n", pIpc->Header.Len);

        DBG_TRACE("waiting responce message from msg queue %i, key 0x%08lx\n", rsp_msqid, rsp_key);
        
        /* Add timer if request hangs */
        if(gettimeofday(&tv, 0))
                return -1;
       
        (void) signal(SIGALRM, sndtimeout);
        memset(&it, 0, sizeof(struct  itimerval));
        it.it_value.tv_sec = pIpc->Header.Timeout/1000;        
        setitimer(ITIMER_REAL, &it, 0);
        
        if (msgsnd(req_msqid, (struct msgbuf*)pmsg, msg_len, IPC_NOWAIT) == -1){
                perror("msgsnd");
                DBG_ERROR("msgsnd failed, msq: %i, key: 0x%08lx, len: %i\n", req_msqid, req_key, msg_len);
                ret = -1;
        }
        else if ( (ret = msgrcv(rsp_msqid, (void*)pIpc_rsp, BUF_SIZE - sizeof(long int), RSP_MSG_TYPE((pIpc->Header.MessageID | IPC_RSP_BASE), pIpc->Header.TID), 0)) == -1){
                perror("msgrcv");
                DBG_ERROR("msgrcv failed, msq: 0x%x\n", rsp_msqid);
                ret = -1; 
        }
        else { //Request succeeded
                char* tmp;
                int len = 0, i = 0;
                memmove(pIpc_rsp, ((MSQMSG*)pIpc_rsp)->ipc_msg, ret);
                
                DBG("received responce: \n");
                DBG("TID: %x\n", pIpc_rsp->Header.TID);
                DBG("Message ID: %x\n", pIpc_rsp->Header.MessageID);
                DBG("Timeout: %li\n", pIpc_rsp->Header.Timeout);
                DBG("Len: %li\n", pIpc_rsp->Header.Len);
                DBG("Status: ");
                if (pIpc_rsp->Status != 0) {
                        DBG("ERROR %li\n", pIpc_rsp->Status);
                        switch(pIpc_rsp->Status){
                            case IPC_STATUS_ERROR:
                                    printf("An error occurred\n");
                                    ret = -1;
                                    break;
                            case IPC_STATUS_NOT_HANDLED:
                                    printf("Command was not handled\n");
                                    ret = -1;
                                    break;
                            case IPC_STATUS_INVALIDPARAM:
                                    printf("Invalid parameters\n");
                                    ret = -1;
                                    break;
                            case IPC_STATUS_FATAL_ERROR:
                                    printf("A fatal error occurred\n");
                                    ret = -1;
                                    break;
                            case IPC_STATUS_PIPE_TIMEOUT:
                            case IPC_STATUS_ACTION_TIMEOUT:
                            case IPC_STATUS_REQUEST_TIMEOUT:
                                    printf("Command timed out\n");
                                    ret = -1;
                                    break;
                            case IPC_STATUS_NOT_IMPLEMENTED:
                                    printf("Not implemented\n");
                                    ret = -1;
                                    break;
                            default:
                                    printf("Unknown error occurred\n");
                                    ret = -1;
                                    break;
                        }
                                    
                                    
                } else {
                        printf("D211: Command succeeded\n");
                        DBG("Status: %li\n", pIpc_rsp->Status);
                }

                len = ret - sizeof(IPCRSPHEADER);
                DBG("DataLen %i\n", len); 
                tmp = (char*)pIpc_rsp->Data;
                for (i = 0; i < len; i++){
                        DBG("%02x ", 0x000000ff&tmp[i]); 
                }
                DBG("\n");

        }

        /* del timer */
        (void) signal(SIGALRM, SIG_DFL);
        it.it_value.tv_sec = 0;
        setitimer(ITIMER_REAL, &it, 0);

        delete (pmsg);
        
        return ret;
}




